//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#include "StdAfx.h"
#include "zlib/zlib.h"
#include "updownclient.h"
#include "partfile.h"
#include "opcodes.h"
#include "packets.h"
#include "emule.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//	members of CUpDownClient
//	which are mainly used for downloading functions 

void CUpDownClient::DrawStatusBar(CDC* dc, RECT* rect, bool onlygreyrect, bool  bFlat)
{ 
	EMULE_TRY

	const COLORREF crBoth=(bFlat) ? RGB(0, 150, 0) : RGB(0, 192, 0);	//Cax2 - same green used for files & clients 
	const COLORREF crNeither=(bFlat) ? RGB(224, 224, 224) : RGB(240, 240, 240); 
	const COLORREF crClientOnly=(bFlat) ? RGB(0, 0, 0) : RGB(95, 95, 95); 
	const COLORREF crPending = RGB(255,208,0);
	const COLORREF crNextPending = RGB(255,255,100);
	//--- xrmb:seetheneed ---
	const COLORREF crMeOnly = RGB(190,190,255); 
	//--- :xrmb ---
	
	uint32 filesize=(reqfile)? reqfile->GetFileSize():1;

	CBarShader statusBar(rect->bottom - rect->top, rect->right - rect->left);
	statusBar.SetFileSize(filesize);
	statusBar.Fill(crNeither);

	if(reqfile)
	{
		if (!onlygreyrect && reqfile && m_abyPartStatus) { 
			_Bvector gettingParts;
			bool isDLing=(m_nDownloadState == DS_DOWNLOADING);
			if (isDLing)	//Cax2 - no unneded calls to showdownloadingparts...
				ShowDownloadingParts(gettingParts);		// Barry - was only showing one part from client, even when reserved bits from 2 parts
			uint32 uEnd; //Cax2 less cpu usage
			for (uint32 i = 0;i != m_nPartCount;i++)
			{ 	
				//--- xrmb:seetheneed --- & Cax2 graphic glitch fix
				if (PARTSIZE*(i+1) > filesize) 
					uEnd = filesize; 
				else 
					uEnd = PARTSIZE*(i+1); 
				if (m_abyPartStatus[i]){ //they have this part
					// do i have it ?
					if  (reqfile->IsComplete(PARTSIZE*i,PARTSIZE*(i+1)-1))
						statusBar.FillRange(PARTSIZE*i,  uEnd, crBoth); 
					else if (isDLing && m_nLastBlockOffset < uEnd && 
														m_nLastBlockOffset >= PARTSIZE*i)
							statusBar.FillRange(PARTSIZE*i,  uEnd, crPending);      
					else if (isDLing && gettingParts[i])	//Cax2 - no more flashes of yellow across all clients...
						statusBar.FillRange(PARTSIZE*i, uEnd, crNextPending);
	    			else
						statusBar.FillRange(PARTSIZE*i, uEnd, crClientOnly);
				}
				else //they don't have a part. Do I have it?
					if (reqfile->IsComplete(PARTSIZE*i,PARTSIZE*(i+1)-1)) 
						statusBar.FillRange(PARTSIZE*i,  uEnd, crMeOnly); 
				//--- :xrmb --- & Cax2 graphic glitch fix
			} 
   		} 
	}
   	statusBar.Draw(dc, rect->left, rect->top, bFlat); 

	EMULE_CATCH
} 


bool CUpDownClient::Compare(CUpDownClient* tocomp){
	EMULE_TRY

	if(HasValidHash() && tocomp->HasValidHash())
		return !memcmp(this->GetUserHash(), tocomp->GetUserHash(), 16);
	if (HasLowID())
		return ((this->GetUserID() == tocomp->GetUserID()) && (GetServerIP() == tocomp->GetServerIP()));
	else
		return (this->GetUserID() == tocomp->GetUserID() || (this->GetIP() && this->GetIP() == tocomp->GetIP()) );

	EMULE_CATCH

	return false;
}

void CUpDownClient::AskForDownload()
{
	EMULE_TRY

	if (theApp.listensocket->TooManySockets() && !(socket && socket->IsConnected()) )
	{
		if (GetDownloadState() != DS_TOOMANYCONNS)
			SetDownloadState(DS_TOOMANYCONNS);

		// Added by Tarod [Dnowo] 
		// Vorlost halved the code: asking time random only at the beginning for each new source 
		if (m_dwLastAskedTime == 0) 
			m_dwLastAskedTime = GetTickCount() - FILEREASKTIME + (uint32)ROUND(((float)rand() / RAND_MAX) * 300000); 

		return;
	}

	m_bUDPPending = false;
	m_dwLastAskedTime = GetTickCount();
	SetDownloadState(DS_CONNECTING);
	TryToConnect();

	EMULE_CATCH
}

bool CUpDownClient::IsSourceRequestAllowed() 
{
	EMULE_TRY

	DWORD dwTickCount = GetTickCount() + CONNECTION_LATENCY;
	int nTimePassedClient = dwTickCount - GetLastSrcAnswerTime();
	int nTimePassedFile   = dwTickCount - reqfile->GetLastAnsweredTime();
	bool bNeverAskedBefore = GetLastSrcReqTime() == 0;

	return (
	         //if client has the correct extended protocol
	         ExtProtocolAvailable() && m_bySourceExchangeVer >= 1 &&
			 //AND if we can send sources
			 !theApp.glob_prefs->IsDisabledXS() &&
	         //AND if we need more sources
	         theApp.glob_prefs->GetMaxSourcePerFileSoft() > reqfile->GetSourceCount() &&
	         //AND if...
	         (
	           //source is not complete and file is rare, allow once every 10 minutes
	           ( !m_bCompleteSource &&
	             ( reqfile->GetSourceCount() - reqfile->GetValidSourcesCount() <= RARE_FILE / 4 ||
			       reqfile->GetSourceCount() <= RARE_FILE * 2
			     ) &&
	             (bNeverAskedBefore || nTimePassedClient > SOURCECLIENTREASK)
	           ) ||
	           // otherwise, allow every 90 minutes, but only if we haven't
	           //   asked someone else in last 10 minutes
			   ( (bNeverAskedBefore || nTimePassedClient > SOURCECLIENTREASK * reqfile->GetCommonFilePenalty()) &&
		         (nTimePassedFile > SOURCECLIENTREASK)
	           )
	         )
	       );

	EMULE_CATCH

	return false;
}

/*
void CUpDownClient::SendFileRequestForUpload(){
	Packet* packet = new Packet(OP_FILEREQUEST,16);
	memcpy(packet->pBuffer,reqfileid,16);
	theApp.uploadqueue->AddUpDataRateMSOverhead(packet->size, 160);
	socket->SendPacket(packet,false);

	packet->opcode = OP_SETREQFILEID;
	socket->SendPacket(packet,true);

	if (GetDownloadState() != DS_NONE) {
		SendFileRequest();
	}
}
*/

void CUpDownClient::SendFileRequest()
{
	EMULE_TRY

	ASSERT(reqfile != NULL);
	if(!reqfile)
		return;
	AddAskedCountDown();
	CMemFile* data = new CMemFile();
	data->Write(reqfile->GetFileHash(),16);
	if( GetExtendedRequestsVersion() > 0 ){
		reqfile->WritePartStatus(data);
	}
	Packet* packet = new Packet(data);
	packet->opcode=OP_FILEREQUEST;
	theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
	socket->SendPacket(packet, true);
	delete data;
	data = new CMemFile();
	data->Write(reqfile->GetFileHash(),16);
	packet = new Packet(data);
	packet->opcode = OP_SETREQFILEID;
	theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
	socket->SendPacket(packet, true);
	delete data;
	if( IsEmuleClient() )
		SetRemoteQueueFull( true );
	SetRemoteQueueRank(0);
	
	if (theApp.glob_prefs->GetMaxSourcePerFileSoft() < reqfile->GetSourceCount() || theApp.glob_prefs->IsDisabledXS())
		return;
	if (theApp.glob_prefs->DisableXSUpTo() && theApp.glob_prefs->XSUpTo() < reqfile->GetSourceCount()) // DonGato - Disable XS for up to X sources
		return;
	if(IsSourceRequestAllowed()) {
		reqfile->SetLastAnsweredTimeTimeout();
		Packet* packet = new Packet(OP_REQUESTSOURCES,16,OP_EMULEPROT);
		memcpy(packet->pBuffer,reqfile->GetFileHash(),16);
		theApp.uploadqueue->AddUpDataOverheadSourceExchange(packet->size);
		socket->SendPacket(packet,true,true);
		//AddDebugLogLine(false, "Send:Source Request User(%s) File(%s)", GetUserName(), reqfile->GetFileName());
	}

	EMULE_CATCH
}

void CUpDownClient::ProcessFileInfo(char* packet,uint32 size)
{
	EMULE_TRY

	CSafeMemFile* data = new CSafeMemFile((BYTE*)packet,size);
	uchar cfilehash[16];
	data->Read(cfilehash,16);
	uint16 namelength;
	data->Read(&namelength,2);
	if (m_pszClientFilename)
		delete[] m_pszClientFilename;
	m_pszClientFilename = new char[namelength+1];
	memset(m_pszClientFilename, 0, namelength+1);
	data->Read(m_pszClientFilename,namelength);	// TODO why does this overwrite the end of the buffer 
							//- because sizeof(uint16) is always 2 
	delete data;
	if ( (!reqfile) || memcmp(cfilehash,reqfile->GetFileHash(),16))
		throw CString(GetResString(IDS_ERR_WRONGFILEID)+ " (ProcessFileInfo)");

	EMULE_CATCH
}

void CUpDownClient::ProcessFileStatus(char* packet,uint32 size){
	EMULE_TRY

	CSafeMemFile data((BYTE*)packet,size);	//TwoBottle Mod (exceptions!)
	uchar cfilehash[16];
	data.Read(cfilehash,16);	
	if ( (!reqfile) || memcmp(cfilehash,reqfile->GetFileHash(),16)){
		//delete data;	//mf	//TwoBottle Mod
		throw CString(GetResString(IDS_ERR_WRONGFILEID)+ " (ProcessFileStatus)");	
	}
	data.Read(&m_nPartCount,2);
	if (m_abyPartStatus){
		delete[] m_abyPartStatus;
		m_abyPartStatus = NULL;
    }
	bool bPartsNeeded = false;
	if (!m_nPartCount){
		m_nPartCount = reqfile->GetPartCount();
		m_abyPartStatus = new uint8[m_nPartCount];
		memset(m_abyPartStatus,1,m_nPartCount);
		bPartsNeeded = true;
		m_bCompleteSource = true;
	}
	else{
		if (reqfile->GetPartCount() != m_nPartCount){
			//delete data;	//mf	//TwoBottle Mod
			throw GetResString(IDS_ERR_WRONGPARTNUMBER);
		}
		m_bCompleteSource = false;
		m_abyPartStatus = new uint8[m_nPartCount];
		uint16 done = 0;
		while (done != m_nPartCount){
			uint8 toread;
			data.Read(&toread,1);
			for (sint32 i = 0;i != 8;i++){
				m_abyPartStatus[done] = ((toread>>i)&1)? 1:0; 	
				if (m_abyPartStatus[done] && !reqfile->IsComplete(done*PARTSIZE,((done+1)*PARTSIZE)-1))
					bPartsNeeded = true;
				done++;
				if (done == m_nPartCount)
					break;
			}
		}
	}

	//theApp.emuledlg->transferwnd.downloadlistctrl.UpdateItem(this);
	UpdateDisplayedInfo();
	reqfile->UpdateAvailablePartsCount();

    CheckForUpPartsInfo(); // obaldin

	if (!bPartsNeeded)
		SetDownloadState(DS_NONEEDEDPARTS);
	else if (reqfile->hashsetneeded){
		Packet* packet = new Packet(OP_HASHSETREQUEST,16);
		memcpy(packet->pBuffer,reqfile->GetFileHash(),16);
		theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
		socket->SendPacket(packet, true, true);
		SetDownloadState(DS_REQHASHSET);
		reqfile->hashsetneeded = false;
	}
	else{
		SetDownloadState(DS_ONQUEUE);
		//delete data;	//TwoBottle Mod
		CSafeMemFile data1(16);	//TwoBottle Mod
		data1.Write(this->reqfile->GetFileHash(),16);
		Packet* packet = new Packet(&data1);
		packet->opcode = OP_STARTUPLOADREQ;
		theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
		socket->SendPacket(packet, true, true);
	}
		
	reqfile->NewSrcPartsInfo();
	//delete data;	//TwoBottle Mod

	EMULE_CATCH
}

bool CUpDownClient::AddRequestForAnotherFile(CPartFile* file)
{
	EMULE_TRY

	if(file == NULL)
		return false;

	for (POSITION pos = m_OtherNoNeeded_list.GetHeadPosition();pos != 0;m_OtherNoNeeded_list.GetNext(pos)){
		if (m_OtherNoNeeded_list.GetAt(pos) == file)
			return false;
	}
	for (POSITION pos = m_OtherRequests_list.GetHeadPosition();pos != 0;m_OtherRequests_list.GetNext(pos)){
		if (m_OtherRequests_list.GetAt(pos) == file)
			return false;
	}
	m_OtherRequests_list.AddTail(file);
	return true;

	EMULE_CATCH

	return false;
}

void CUpDownClient::SetDownloadState(uint8 byNewState)
{
	EMULE_TRY

	if (m_nDownloadState != byNewState){
		if (m_nDownloadState == DS_DOWNLOADING ){
			m_nDownloadState = byNewState;
			for (POSITION pos = m_DownloadBlocks_list.GetHeadPosition();pos != 0;m_DownloadBlocks_list.GetNext(pos)){
				Requested_Block_Struct* cur_block = m_DownloadBlocks_list.GetAt(pos);
				reqfile->RemoveBlockFromList(cur_block->StartOffset,cur_block->EndOffset);
				delete m_DownloadBlocks_list.GetAt(pos);
			}
			m_DownloadBlocks_list.RemoveAll();

			for (POSITION pos = m_PendingBlocks_list.GetHeadPosition();pos != 0;m_PendingBlocks_list.GetNext(pos))
			{
				Pending_Block_Struct *pending = m_PendingBlocks_list.GetAt(pos);
				if (reqfile)
					reqfile->RemoveBlockFromList(pending->block->StartOffset, pending->block->EndOffset);
				
				delete pending->block;
				// Not always allocated
				if (pending->zStream) delete pending->zStream;
				delete pending;
			}
			m_PendingBlocks_list.RemoveAll();
			m_nDownDatarate = 0;
			if (byNewState == DS_NONE){
				if (m_abyPartStatus)
					delete[] m_abyPartStatus;
				m_abyPartStatus = 0;
			}
			m_LastPartAsked = 0xffff; //jicxicmic
			if (socket && byNewState != DS_ERROR)
					socket->DisableDownloadLimit();
		}
		m_nDownloadState = byNewState;
		if( GetDownloadState() == DS_DOWNLOADING ){
			if ( IsEmuleClient() )
			SetRemoteQueueFull(false);
			SetRemoteQueueRank(1);
			SetAskedCountDown(0);
		}
		//theApp.emuledlg->transferwnd.downloadlistctrl.UpdateItem(this);
		UpdateDisplayedInfo(true);
	}

	EMULE_CATCH
}

void CUpDownClient::ProcessHashSet(char* packet,uint32 size)
{
	EMULE_TRY

	if ( (!reqfile) || memcmp(packet,reqfile->GetFileHash(),16))
		throw CString(GetResString(IDS_ERR_WRONGFILEID)+" (ProcessHashSet)");	
	CSafeMemFile* data = new CSafeMemFile((BYTE*)packet,size);
	if (reqfile->LoadHashsetFromFile(data,true))
	{
		theApp.knownfiles->SafeAddKFile(reqfile, true);   // DonkeyDoctor 
	}
	else{
		reqfile->hashsetneeded = true;
		delete data;	//mf
		throw GetResString(IDS_ERR_BADHASHSET);
	}
	SetDownloadState(DS_ONQUEUE);
	delete data;
	data = new CSafeMemFile(16);
	data->Write(this->reqfile->GetFileHash(),16);
	Packet* opacket = new Packet(data);
	opacket->opcode = OP_STARTUPLOADREQ;
	theApp.uploadqueue->AddUpDataOverheadFileRequest(opacket->size);
	socket->SendPacket(opacket, true, true);
	delete data;

	EMULE_CATCH
}

void CUpDownClient::SendBlockRequests()
{
	EMULE_TRY

	m_dwLastBlockReceived = GetTickCount();
	if (!reqfile)
		return;
	//jicxicmic - get blocks from the same chunk first
	if (m_DownloadBlocks_list.IsEmpty())
	{
		//first time we ask?
		if (m_LastPartAsked == 0xffff)
			m_LastPartAsked = reqfile->GetNextRequestedBlock(this);


		// Barry - instead of getting 3, just get how many is needed
		uint16 count, savedcount;
		savedcount = count = 3 - m_PendingBlocks_list.GetCount();
		Requested_Block_Struct** toadd = new Requested_Block_Struct*[count];
		if (m_LastPartAsked != 0xffff)		//previous active part ?
		{
			reqfile->GetGapsInPart(this, m_LastPartAsked, toadd, &count);
			if (count == 0)		//finished part ?
			{
				m_LastPartAsked = reqfile->GetNextRequestedBlock(this); //get next part
				if (m_LastPartAsked != 0xffff)		//more parts ?
				{
					count = savedcount;
					reqfile->GetGapsInPart(this, m_LastPartAsked, toadd, &count);
				}
			}
		}
		else
			count=0;

		for (int i = 0; i != count; i++)
			m_DownloadBlocks_list.AddTail(toadd[i]);

		delete[] toadd;
		if (count == 0)
			m_LastPartAsked = 0xffff;
	}
	//jicxicmic - get blocks from the same chunk first



	// Barry - Why are unfinished blocks requested again, not just new ones?

	while (m_PendingBlocks_list.GetCount() < 3 && !m_DownloadBlocks_list.IsEmpty()){
		Pending_Block_Struct* pblock = new Pending_Block_Struct;
		pblock->block = m_DownloadBlocks_list.RemoveHead();
		pblock->zStream = NULL;
		pblock->totalUnzipped = 0;
		m_PendingBlocks_list.AddTail(pblock);
	}
	if (m_PendingBlocks_list.IsEmpty()){
		Packet* packet = new Packet(OP_CANCELTRANSFER,0);
		theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
		socket->SendPacket(packet,true,true);
		//SetDownloadState(DS_NONEEDEDPARTS);
		AskForDownload(); //<<--enkeyDEV(Ottavio84) - reask for file
		return;
	}
	Packet* packet = new Packet(OP_REQUESTPARTS,40);
	CMemFile* data = new CMemFile((BYTE*)packet->pBuffer,40);
	data->Write(reqfile->GetFileHash(),16);
	POSITION pos = m_PendingBlocks_list.GetHeadPosition();
	uint32 null = 0;
	Requested_Block_Struct* block;
	for (uint32 i = 0; i != 3; i++){
		if (pos){
			block = m_PendingBlocks_list.GetAt(pos)->block;
			m_PendingBlocks_list.GetNext(pos);
			data->Write(&block->StartOffset,4);
		}
		else
			data->Write(&null,4);
	}
	pos = m_PendingBlocks_list.GetHeadPosition();
	for (uint32 i = 0; i != 3; i++){
		if (pos){
			block = m_PendingBlocks_list.GetAt(pos)->block;
			m_PendingBlocks_list.GetNext(pos);
			uint32 endpos = block->EndOffset+1;
			data->Write(&endpos,4);
		}
		else
			data->Write(&null,4);
	}
	delete data;
	theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
	socket->SendPacket(packet,true,true);

	EMULE_CATCH
}

/* Barry - Originally this only wrote to disk when a full 180k block 
           had been received from a client, and only asked for data in 
		   180k blocks.

		   This meant that on average 90k was lost for every connection
		   to a client data source. That is a lot of wasted data.

		   To reduce the lost data, packets are now written to a buffer
		   and flushed to disk regularly regardless of size downloaded.
		   This includes compressed packets.

		   Data is also requested only where gaps are, not in 180k blocks.
		   The requests will still not exceed 180k, but may be smaller to
		   fill a gap.
*/
void CUpDownClient::ProcessBlockPacket(char* packet, uint32 size, bool packed)
{
	EMULE_TRY

	// Ignore if no data required
	if (!(GetDownloadState() == DS_DOWNLOADING || GetDownloadState() == DS_NONEEDEDPARTS))
		return;

	const int HEADER_SIZE = 24;

	// Update stats
	m_dwLastBlockReceived = ::GetTickCount();

	// Read data from packet
	CSafeMemFile* data = new CSafeMemFile((BYTE*)packet,size);
	uchar fileID[16];
	data->Read(fileID, 16);

	// Check that this data is for the correct file
	if ( (!reqfile) || memcmp(packet,reqfile->GetFileHash(),16))
	{
		delete data;
		throw CString(GetResString(IDS_ERR_WRONGFILEID)+" (ProcessBlockPacket)");
	}

	// Find the start & end positions, and size of this chunk of data
	uint32 nStartPos;
	uint32 nEndPos;
	uint32 nBlockSize = 0;
	data->Read(&nStartPos,4);
	if (packed)
	{
		data->Read(&nBlockSize,4);
		nEndPos = nStartPos + (size - HEADER_SIZE);
		usedcompressiondown = true;
	}
	else
		data->Read(&nEndPos,4);
	delete data;
		
	// Check that packet size matches the declared data size + header size (24)
	if ( size != ((nEndPos - nStartPos) + HEADER_SIZE))
		throw CString(GetResString(IDS_ERR_BADDATABLOCK)+" (ProcessBlockPacket)");

	// Move end back one, should be inclusive 
	theApp.UpdateReceivedBytes(size - HEADER_SIZE);
	uint32 DownDataRateMS = size - HEADER_SIZE;
	credits->AddDownloaded(size - HEADER_SIZE);
	nEndPos--;

	m_nTransferedDown += DownDataRateMS;

    DWORD curTick = GetTickCount();
	if(m_Average_DL_tick_list.empty() || curTick - m_Average_DL_tick_list.front()>= 500)	//Cax2 - update no faster than every .5 sec
	{
		m_AverageDDR_list.push_front(m_nTransferedDown);
		m_Average_DL_tick_list.push_front(curTick);

		// don't save more than 40 secs of data
		while(curTick-m_Average_DL_tick_list.back() > 40000) {
   			m_AverageDDR_list.pop_back();
			m_Average_DL_tick_list.pop_back();
		}
	}
	// Loop through to find the reserved block that this is within
	Pending_Block_Struct* cur_block;
	for (POSITION pos = m_PendingBlocks_list.GetHeadPosition(); pos != NULL; m_PendingBlocks_list.GetNext(pos))
	{
		cur_block = m_PendingBlocks_list.GetAt(pos);
		//Cax2: I'm not happy with this whole if clause
		if ((cur_block->block->StartOffset <= nStartPos) && (cur_block->block->EndOffset >= nStartPos))
		{
			// Found reserved block

			// Remember this start pos, used to draw part downloading in list
			m_nLastBlockOffset = nStartPos;  

			// Occasionally packets are duplicated, no point writing it twice
			// This will be 0 in these cases, or the length written otherwise
			uint32 lenWritten = 0;

			// Handle differently depending on whether packed or not
			if (!packed)
			{
				// Write to disk (will be buffered in part file class)
				lenWritten = reqfile->WriteToBuffer(size - HEADER_SIZE, 
													(BYTE *) (packet + HEADER_SIZE),
													nStartPos,
													nEndPos,
													cur_block->block );
			}
			else // Packed
			{
				// Create space to store unzipped data, the size is only an initial guess, will be resized in unzip() if not big enough
				uint32 lenUnzipped = (size * 2); 
				// Don't get too big
				if (lenUnzipped > (BLOCKSIZE + 300))
					lenUnzipped = (BLOCKSIZE + 300);
				BYTE *unzipped = new BYTE[lenUnzipped];

				// Try to unzip the packet
				int result = unzip(cur_block, (BYTE *)(packet + HEADER_SIZE), (size - HEADER_SIZE), &unzipped, &lenUnzipped);
				if (result == Z_OK)
				{
					// Write any unzipped data to disk
					if (lenUnzipped > 0)
					{
						// Use the current start and end positions for the uncompressed data
						nStartPos = cur_block->block->StartOffset + cur_block->totalUnzipped - lenUnzipped;
						nEndPos = cur_block->block->StartOffset + cur_block->totalUnzipped - 1;
					
						// Write uncompressed data to file
						lenWritten = reqfile->WriteToBuffer(size - HEADER_SIZE,
															unzipped,
															nStartPos,
															nEndPos,
															cur_block->block );
					}
				}
				else
				{
					AddDebugLogLine(false, GetResString(IDS_ERR_CORRUPTCOMPRPKG), reqfile->GetFileName(), result);
					reqfile->RemoveBlockFromList(cur_block->block->StartOffset, cur_block->block->EndOffset);
				}
				delete [] unzipped;
			}

			// These checks only need to be done if any data was written
			if (lenWritten > 0)
			{
				//m_nTransferedDown += lenWritten;

				// If finished reserved block
				if (nEndPos == cur_block->block->EndOffset)
				{
					reqfile->RemoveBlockFromList(cur_block->block->StartOffset, cur_block->block->EndOffset);
					delete cur_block->block;
					// Not always allocated
					if (cur_block->zStream) delete cur_block->zStream;
					delete cur_block;
					m_PendingBlocks_list.RemoveAt(pos);

					// Request next block
					SendBlockRequests();	
				}
			}

			// Stop looping and exit method
			return;
		}
	}
	EMULE_CATCH
}

int CUpDownClient::unzip(Pending_Block_Struct *block, BYTE *zipped, uint32 lenZipped, BYTE **unzipped, uint32 *lenUnzipped, bool recursive)
{
  int err = Z_DATA_ERROR;
  EMULE_TRY
	// Save some typing
	z_stream *zS = block->zStream;

	// Is this the first time this block has been unzipped
	if (zS == NULL)
	{
		// Create stream
		block->zStream = new z_stream;
		zS = block->zStream;

		// Initialise stream values
		zS->zalloc = (alloc_func)0;
		zS->zfree = (free_func)0;
		zS->opaque = (voidpf)0;

		// Set output data streams, do this here to avoid overwriting on recursive calls
		zS->next_out = (*unzipped);
		zS->avail_out = (*lenUnzipped);

		// Initialise the z_stream
		err = inflateInit(zS);
		if (err != Z_OK)
			return err;
	}

	// Use whatever input is provided
	zS->next_in  = zipped;
	zS->avail_in = lenZipped;

	// Only set the output if not being called recursively
	if (!recursive)
	{
		zS->next_out = (*unzipped);
		zS->avail_out = (*lenUnzipped);
	}

	// Try to unzip the data
	err = inflate(zS, Z_SYNC_FLUSH);

	// Is zip finished reading all currently available input and writing all generated output
	if (err == Z_STREAM_END)
	{
		// Finish up
		err = inflateEnd(zS);
		if (err != Z_OK)
			return err;

		// Got a good result, set the size to the amount unzipped in this call (including all recursive calls)
		(*lenUnzipped) = (zS->total_out - block->totalUnzipped);
		block->totalUnzipped = zS->total_out;
	}
	else if ((err == Z_OK) && (zS->avail_out == 0))
	{
		// Output array was not big enough, call recursively until there is enough space

		// What size should we try next
		uint32 newLength = (*lenUnzipped) *= 2;
		if (newLength == 0)
			newLength = lenZipped * 2;

		// Copy any data that was successfully unzipped to new array
		BYTE *temp = new BYTE[newLength];
		memcpy(temp, (*unzipped), (zS->total_out - block->totalUnzipped));
		delete [] (*unzipped);
		(*unzipped) = temp;
		(*lenUnzipped) = newLength;

		// Position stream output to correct place in new array
		zS->next_out = (*unzipped) + (zS->total_out - block->totalUnzipped);
		zS->avail_out = (*lenUnzipped) - (zS->total_out - block->totalUnzipped);

		// Try again
		err = unzip(block, zS->next_in, zS->avail_in, unzipped, lenUnzipped, true);
	}
	else if ((err == Z_OK) && (zS->avail_in == 0))
	{
		// All available input has been processed, everything ok.
		// Set the size to the amount unzipped in this call (including all recursive calls)
		(*lenUnzipped) = (zS->total_out - block->totalUnzipped);
		block->totalUnzipped = zS->total_out;
		//Cax2 - not needed here, it stops downloads prematurely
		//inflateEnd(zS);
	}
	else
	{
		// Should not get here unless input data is corrupt
		AddDebugLogLine(false,"Unexpected zip error");
		//DebugBreak(); I removed this so that we could let the client run for more the five minutes.. Barry needs to see if there is or isn't a preoblem here..
		err=inflateEnd(zS);
	}

	if (err != Z_OK)
		(*lenUnzipped) = 0;
  EMULE_CATCH
  return err;
}

sint32 CUpDownClient::CalculateDownloadRate(){
	EMULE_TRY
    if ((GetTickCount() - m_dwLastBlockReceived) > DOWNLOADTIMEOUT){
		Packet* packet = new Packet(OP_CANCELTRANSFER,0);
		theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
		socket->SendPacket(packet,true,true);
		SetDownloadState(DS_ONQUEUE);

        m_nDownDatarate = 0;
    } else {
        DWORD curTick = GetTickCount();

        if(curTick-m_lastCalculatedDownloadRate >= MINWAIT_BEFORE_CALCULATE_DL_RATE) {
            m_lastCalculatedDownloadRate = curTick;

            // Calculate average datarate

            // Present this slot's speed as 0 unless there's a good chunk
            // of stored statistics for it. If there's to little data, the
            // speed will be far off, so we might as well hide it in those
            // cases.
            if(m_AverageDDR_list.size() >= 2) {
	            m_nDownDatarate = (uint32)(((float)(m_AverageDDR_list.front()-m_AverageDDR_list.back()))*1000 / (m_Average_DL_tick_list.front()-m_Average_DL_tick_list.back()));
            } else {
                m_nDownDatarate = 0;
            }
        } else {
            // uses cached m_nDownDatarate
        }
    }
	//Cax2 - the gain was minimal, not worth keeping that...
	UpdateDisplayedInfo();
    return m_nDownDatarate;

	EMULE_CATCH

	return 0;
}

uint16 CUpDownClient::GetAvailablePartCount()
{
	EMULE_TRY

	uint16 result = 0;
	for (int i = 0;i != m_nPartCount;i++){
		if (IsPartAvailable(i))
			result++;
	}
	return result;

	EMULE_CATCH

	return 0;
}

void CUpDownClient::SetRemoteQueueRank(uint16 nr)
{
	EMULE_TRY

	m_nRemoteQueueRank = nr;
	//theApp.emuledlg->transferwnd.downloadlistctrl.UpdateItem(this);
	UpdateDisplayedInfo();

	EMULE_CATCH
}

bool CUpDownClient::SwapToAnotherFile(bool bIgnoreNoNeeded, bool bManualChange)
{
	EMULE_TRY
	if (GetDownloadState() == DS_DOWNLOADING)
		return false;

	// Modified by Tarod (get first high priority files)
	POSITION SwapToPos = 0;
	CPartFile* SwapTo = NULL;
	CPartFile* cur_file;		//Cax2 CPU optimization
	if (!m_OtherRequests_list.IsEmpty()){
		for (POSITION pos = m_OtherRequests_list.GetHeadPosition();pos != 0;m_OtherRequests_list.GetNext(pos)){
			cur_file = m_OtherRequests_list.GetAt(pos);
			if (cur_file != reqfile && theApp.downloadqueue->IsPartFile(cur_file) && (cur_file->GetStatus(false) == PS_READY || cur_file->GetStatus(false) == PS_EMPTY) ){
				if (!SwapTo || (!bManualChange && SwapTo->GetPriority() < cur_file->GetPriority()))
				{
					SwapTo = cur_file;
					SwapToPos = pos ;
				}
			}
		}
		if (SwapToPos != 0) m_OtherRequests_list.RemoveAt(SwapToPos);
	}
	// Modified by Tarod (get first high priority files)
	if (!SwapTo && bIgnoreNoNeeded){
		for (POSITION pos = m_OtherNoNeeded_list.GetHeadPosition();pos != 0;m_OtherNoNeeded_list.GetNext(pos)){
			cur_file = m_OtherNoNeeded_list.GetAt(pos);
			if (cur_file != reqfile && theApp.downloadqueue->IsPartFile(cur_file) && (cur_file->GetStatus(false) == PS_READY || cur_file->GetStatus(false) == PS_EMPTY) ){
				if (!SwapTo || (!bManualChange && SwapTo->GetPriority() < cur_file->GetPriority()))
				{
					SwapTo = cur_file;
					SwapToPos = pos ;
				}
			}
		}
		if (SwapToPos != 0) m_OtherNoNeeded_list.RemoveAt(SwapToPos);
	}
	if (SwapTo){
		m_OtherNoNeeded_list.AddHead(reqfile);
		theApp.downloadqueue->RemoveSource(this);
		m_nRemoteQueueRank = 0;
		if (m_abyPartStatus){
			delete[] m_abyPartStatus;
			m_abyPartStatus = 0;
		}
		m_dwLastAskedTime = 0;
		m_iRate=0;
		m_strComment="";
		// Lord KiRon A4AF bugfix + Cax2 bugfix: hanging sources!
      	if (bManualChange)
		{
			if(!theApp.downloadqueue->CheckAndAddKnownSource(SwapTo,this)) 
				return false;
		}else{
			theApp.downloadqueue->CheckAndAddKnownSource(SwapTo,this);
		}
      	// Lord KiRon - end
      	return true;
	}
	EMULE_CATCH
	return false;
}

void CUpDownClient::UDPReaskACK(uint16 nNewQR)
{
	EMULE_TRY
	m_bUDPPending = false;
	SetRemoteQueueRank(nNewQR);
	m_dwLastAskedTime = ::GetTickCount();

	EMULE_CATCH
}

void CUpDownClient::UDPReaskFNF()
{
	EMULE_TRY
	m_bUDPPending = false;
	AddDebugLogLine(false,CString("UDP ANSWER FNF : %s"),GetUserName());
	theApp.downloadqueue->RemoveSource(this);
	if (!socket)
		Disconnected();
	EMULE_CATCH
}

void CUpDownClient::UDPReaskForDownload()
{
	EMULE_TRY
	ASSERT ( reqfile );
	if (!reqfile || m_bUDPPending)
		return;


	// Maverick Disable UDP when connecting to a PROXY
    ProxySettings proxy = theApp.glob_prefs->GetProxy();
	if(m_byEmuleVersion >= 0x23 && m_nUDPPort != 0 && theApp.glob_prefs->GetUDPPort() != 0 &&
	   !theApp.serverconnect->IsLowID() && !HasLowID() && !(socket && socket->IsConnected()) && (!proxy.UseProxy)) {

			//don't use udp to ask for sources
			if(IsSourceRequestAllowed())
				return;
			m_bUDPPending = true;
			Packet* response = new Packet(OP_REASKFILEPING,16,OP_EMULEPROT);
			memcpy(response->pBuffer,reqfile->GetFileHash(),16);
	        theApp.uploadqueue->AddUpDataOverheadFileRequest(response->size);
			theApp.clientudp->SendPacket(response,GetIP(),GetUDPPort());
		}

   EMULE_CATCH
}

// Barry - Sets string to show parts downloading, eg NNNYNNNNYYNYN
void CUpDownClient::ShowDownloadingParts(_Bvector& parts)
{
	EMULE_TRY

	parts.resize(m_nPartCount);
	for (POSITION pos = m_PendingBlocks_list.GetHeadPosition();pos;)
		parts[m_PendingBlocks_list.GetNext(pos)->block->StartOffset / PARTSIZE]=true;

	EMULE_CATCH
}

void CUpDownClient::UpdateDisplayedInfo(boolean force) {
    DWORD curTick = ::GetTickCount();

    if(force || curTick-m_lastRefreshedDLDisplay > MINWAIT_BEFORE_DLDISPLAY_WINDOWUPDATE+(uint32)(rand()*1000/RAND_MAX)) {
	    theApp.emuledlg->transferwnd.downloadlistctrl.UpdateItem(this);
        m_lastRefreshedDLDisplay = curTick;
    }
}

// New functions by Mr.Fry
uint16 CUpDownClient::GetAvailableOfNeededPartsCount()
{
	EMULE_TRY

	uint16 count = 0;
	if(m_abyPartStatus != NULL && m_nPartCount > 0){
		for(int i=0; i < m_nPartCount ; i++)
			if (m_abyPartStatus[i] && !reqfile->IsComplete(i*PARTSIZE,((i+1)*PARTSIZE)-1))
				count++;
		//TRACE("Counted %u Chunks (parts) that %s has and I need (to download)",count,GetUserName());
	}
	return count;

	EMULE_CATCH

	return 0;
}

// Amdribant
void CUpDownClient::HandoverToAnotherFile(CPartFile* file)		//Cax2 - bugfixed A4AF
{ 
	EMULE_TRY

	if(file == NULL||GetDownloadState() == DS_DOWNLOADING)
		return;
	CSingleLock sLock(&theApp.knownfiles->list_mut,true);		//Cax2 - proteceted!
	 //Cax2 - downloadlist bugfix - removed unnecessary code...
	POSITION pos = m_OtherRequests_list.Find(file); 
	if (pos) { 
      		m_OtherRequests_list.RemoveAt(pos); 
	} 

	//Cax2 - attach them properly
	pos=reqfile->srclists[sourcesslot].Find(this);
	if (pos)
			reqfile->srclists[sourcesslot].RemoveAt(pos);
	CPartFile* cur_file=(CPartFile*) &reqfile;
	m_OtherRequests_list.AddHead(cur_file);
	reqfile=file;
	cur_file->UpdateAvailablePartsCount();
	if (m_abyPartStatus)
	{
		delete[] m_abyPartStatus;
		m_abyPartStatus = 0;
	}

	UpdateDisplayedInfo(true);
	
	if (!file->srclists[sourcesslot].Find(this))
		file->srclists[sourcesslot].AddTail(this);

	m_iRate=0;
	m_strComment="";
	m_dwLastAskedTime = 0;
	file->UpdateAvailablePartsCount();
	EMULE_CATCH
 } 
