#include "StdAfx.h"
#include "emule.h"
#include "webserver.h"
#include "XBMDraw.h"
#include "ED2KLink.h"
#include "MD5Sum.h"
#include <stdlib.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


#define HTTPInit "Server: eMule\r\nPragma: no-cache\r\nExpires: 0\r\nCache-Control: no-cache, no-store, must-revalidate\r\nConnection: close\r\nContent-Type: text/html\r\n"
#define HTTPInitGZ "Server: eMule\r\nPragma: no-cache\r\nExpires: 0\r\nCache-Control: no-cache, no-store, must-revalidate\r\nConnection: close\r\nContent-Type: text/html\r\nContent-Encoding: gzip\r\n"

#define WEB_SERVER_TEMPLATES_VERSION	1

static dwThread = 0;

CWebServer::CWebServer(void)
{
	m_Params.bShowUploadQueue = false;

	m_Params.DownloadSort = DOWN_SORT_NAME;
	m_Params.bDownloadSortReverse = false;
	m_Params.ServerSort = SERVER_SORT_NAME;
	m_Params.bServerSortReverse = false;
	m_Params.SharedSort = SHARED_SORT_NAME;
	m_Params.bSharedSortReverse = false;

	dwThread = GetCurrentThreadId();

	srand((unsigned)time(NULL));

	EMULE_TRY

	//m_bServerWorking = theApp.glob_prefs->GetHTTPIsEnabled();
	m_bServerWorking = false;	//Cax2 - this enables webserver  notification at startup

	//ReloadTemplates(); //EC - No need to load templates here.They will be loaded in StartServer()

	//StartSockets(this); // EC - No need to start sockets here. They will be started in StartServer()

	EMULE_CATCH
}

void CWebServer::ReloadTemplates()
{
	EMULE_TRY

	CString sFile = theApp.glob_prefs->GetAppDir() + CString("eMule.tmpl");
	CStdioFile file;
	if(file.Open(sFile, CFile::modeRead|CFile::typeText))
	{
		CString sAll;
		for(;;)
		{
			CString sLine;
			if(!file.ReadString(sLine))
				break;

			sAll += sLine + "\n";
		}
		file.Close();

		CString sVersion = _LoadTemplate(sAll,"TMPL_VERSION");
		long lVersion = atol(sVersion);
		if(lVersion < WEB_SERVER_TEMPLATES_VERSION)
		{
			if(theApp.glob_prefs->GetHTTPIsEnabled() || m_bServerWorking)
			AddLogLine(true,"Can't load templates: replace file " + sFile + " with a newer version!");
			//EC -  No need for webserver to run if there's nothing to show
			if (m_bServerWorking)
				StopSockets(); 
            m_bServerWorking = false; 
			theApp.glob_prefs->SetHTTPIsEnabled(false);
			//EC - ends
		}
		else
		{
			m_Templates.sHeader = _LoadTemplate(sAll,"TMPL_HEADER");
			m_Templates.sHeaderMetaRefresh = _LoadTemplate(sAll,"TMPL_HEADER_META_REFRESH");
			m_Templates.sHeaderStylesheet = _LoadTemplate(sAll,"TMPL_HEADER_STYLESHEET");
			m_Templates.sFooter = _LoadTemplate(sAll,"TMPL_FOOTER");
			m_Templates.sServerList = _LoadTemplate(sAll,"TMPL_SERVER_LIST");
			m_Templates.sServerLine = _LoadTemplate(sAll,"TMPL_SERVER_LINE");
			m_Templates.sTransferImages = _LoadTemplate(sAll,"TMPL_TRANSFER_IMAGES");
			m_Templates.sTransferList = _LoadTemplate(sAll,"TMPL_TRANSFER_LIST");
			m_Templates.sTransferDownHeader = _LoadTemplate(sAll,"TMPL_TRANSFER_DOWN_HEADER");
			m_Templates.sTransferDownFooter = _LoadTemplate(sAll,"TMPL_TRANSFER_DOWN_FOOTER");
			m_Templates.sTransferDownLine = _LoadTemplate(sAll,"TMPL_TRANSFER_DOWN_LINE");
			m_Templates.sTransferDownLineGood = _LoadTemplate(sAll,"TMPL_TRANSFER_DOWN_LINE_GOOD");
			m_Templates.sTransferUpHeader = _LoadTemplate(sAll,"TMPL_TRANSFER_UP_HEADER");
			m_Templates.sTransferUpFooter = _LoadTemplate(sAll,"TMPL_TRANSFER_UP_FOOTER");
			m_Templates.sTransferUpLine = _LoadTemplate(sAll,"TMPL_TRANSFER_UP_LINE");
			m_Templates.sTransferUpQueueShow = _LoadTemplate(sAll,"TMPL_TRANSFER_UP_QUEUE_SHOW");
			m_Templates.sTransferUpQueueHide = _LoadTemplate(sAll,"TMPL_TRANSFER_UP_QUEUE_HIDE");
			m_Templates.sTransferUpQueueLine = _LoadTemplate(sAll,"TMPL_TRANSFER_UP_QUEUE_LINE");
			m_Templates.sTransferBadLink = _LoadTemplate(sAll,"TMPL_TRANSFER_BAD_LINK");
			m_Templates.sDownloadLink = _LoadTemplate(sAll,"TMPL_DOWNLOAD_LINK");
			m_Templates.sSharedList = _LoadTemplate(sAll,"TMPL_SHARED_LIST");
			m_Templates.sSharedLine = _LoadTemplate(sAll,"TMPL_SHARED_LINE");
			m_Templates.sSharedLineChanged = _LoadTemplate(sAll,"TMPL_SHARED_LINE_CHANGED");
			m_Templates.sGraphs = _LoadTemplate(sAll,"TMPL_GRAPHS");
			m_Templates.sLog = _LoadTemplate(sAll,"TMPL_LOG");
			m_Templates.sServerInfo = _LoadTemplate(sAll,"TMPL_SERVERINFO");
			m_Templates.sDebugLog = _LoadTemplate(sAll,"TMPL_DEBUGLOG");
			m_Templates.sStats = _LoadTemplate(sAll,"TMPL_STATS");
			m_Templates.sPreferences = _LoadTemplate(sAll,"TMPL_PREFERENCES");
			m_Templates.sLogin = _LoadTemplate(sAll,"TMPL_LOGIN");
			m_Templates.sConnectedServer = _LoadTemplate(sAll,"TMPL_CONNECTED_SERVER");
			m_Templates.sAddServerBox = _LoadTemplate(sAll,"TMPL_ADDSERVERBOX");
			m_Templates.sWebSearch = _LoadTemplate(sAll,"TMPL_WEBSEARCH");
		}
	}
	else
        if(m_bServerWorking)
		{
			AddLogLine(true,"Can't load templates: Can't open file " + sFile);
			//EC -  No need for webserver to run if there's nothing to show
			StopSockets(); 
            m_bServerWorking = false; 
			theApp.glob_prefs->SetHTTPIsEnabled(false);
			//EC - ends
		}

	EMULE_CATCH
}

CWebServer::~CWebServer(void)
{
	if (m_bServerWorking) // EC
		 StopSockets();
}

CString CWebServer::_LoadTemplate(CString sAll, CString sTemplateName)
{
	EMULE_TRY

	CString sRet = "";
	int nStart = sAll.Find("<--" + sTemplateName + "-->");
	int nEnd = sAll.Find("<--" + sTemplateName + "_END-->");
	if(nStart != -1 && nEnd != -1)
	{
		nStart += sTemplateName.GetLength() + 7;
		sRet = sAll.Mid(nStart, nEnd - nStart - 1);
	}

	if(sRet.IsEmpty())
	{
		if (sTemplateName=="TMPL_VERSION")
			AddLogLine(true,"Can't find template version number!\nPlease replace eMule.tmpl with a newer version!");
			AddDebugLogLine(false,"Failed to load template " + sTemplateName);
	}
	return sRet;

	EMULE_CATCH

	return "";
}

void CWebServer::RestartServer()	//Cax2 - restarts the server with the new port settings
{
	// StopSockets();  //EC - Why stop sockets if they havent start
	if (m_bServerWorking)	
	{
		StopSockets(); //EC - Stop sockets if they have been started
		StartSockets(this);
	}
}

void CWebServer::StartServer(void)
{
	EMULE_TRY

	if(m_bServerWorking != theApp.glob_prefs->GetHTTPIsEnabled())
		{
			m_bServerWorking = theApp.glob_prefs->GetHTTPIsEnabled();
			//EC Starts
			//RestartServer(); 
			StartSockets(this); 
			ReloadTemplates();
			//EC Ends
		}
	else
		return;

	if(theApp.glob_prefs->GetHTTPIsEnabled() && m_bServerWorking) // EC
		AddLogLine(false,"%s: %s",_GetPlainResString(IDS_PW_HTTPD), _GetPlainResString(IDS_ENABLED).MakeLower());
	else
		AddLogLine(false,"%s: %s",_GetPlainResString(IDS_PW_HTTPD), _GetPlainResString(IDS_DISABLED).MakeLower());

	EMULE_CATCH
}

void CWebServer::_RemoveServer(CString sIP)
{
	EMULE_TRY

	for (POSITION pos=theApp.serverlist->list.GetHeadPosition();pos != 0;theApp.serverlist->list.GetNext(pos))
	{
		CServer* cur_server =  theApp.serverlist->list.GetAt(pos);
		if( sIP == CString(cur_server->GetAddress()))
		{
			theApp.emuledlg->SendMessage(WEB_REMOVE_SERVER, (WPARAM)cur_server, NULL);
			break;
		}
	}

	EMULE_CATCH

}
void CWebServer::_SetSharedFilePriority(CString hash, uint8 priority)
{
	EMULE_TRY

	for (POSITION pos = theApp.sharedfiles->m_Files_map.GetStartPosition();pos != 0;)
	{
		CCKey bufKey;
		CKnownFile* cur_file;
		theApp.sharedfiles->m_Files_map.GetNextAssoc(pos,bufKey,cur_file);
		CString cur_hash;
		cur_hash = _GetStringHash(cur_file->GetFileHash());
		if(cur_hash==hash)
		{
			if(priority >= 0 && priority < 5)
			{
				cur_file->SetPriority(priority);
				break;
			}
			else
			if(priority == 5 && cur_file->IsPartFile())
			{
				cur_file->SetAutoPriority(true);
				cur_file->UpdateUploadAutoPriority(); 
				break;
			}
		}
	}

	EMULE_CATCH

}
void CWebServer::AddStatsLine(UpDown line)
{
	EMULE_TRY

	m_Params.PointsForWeb.Add(line);
	if(m_Params.PointsForWeb.GetCount() > WEB_GRAPH_WIDTH)
		m_Params.PointsForWeb.RemoveAt(0);

	EMULE_CATCH
}

CString CWebServer::_SpecialChars(CString str) 
{
	EMULE_TRY

	str.Replace("&","&amp;");
	str.Replace("<","&lt;");
	str.Replace(">","&gt;");
	str.Replace("\"","&quot;");
	return str;

	EMULE_CATCH

	return "";
}

void CWebServer::SetUserCount(uint32 nUsers)
{
	m_Params.nUsers = nUsers;
}

void CWebServer::_ConnectToServer(CString sIP)
{
	EMULE_TRY

	for (POSITION pos=theApp.serverlist->list.GetHeadPosition();pos != 0;theApp.serverlist->list.GetNext(pos))
	{
		CServer* cur_file =  theApp.serverlist->list.GetAt(pos);
		if(sIP == CString(cur_file->GetAddress()))
		{
			theApp.emuledlg->SendMessage(WEB_CONNECT_TO_SERVER, (WPARAM)cur_file, NULL);
			break;
		}
	}

	EMULE_CATCH
}

void CWebServer::ProcessURL(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return;

	if ( theApp.glob_prefs->IsNTBased() )
	{
		SetThreadLocale(theApp.glob_prefs->GetLanguageID());
	}
	CoInitialize(NULL);

	bool isUseGzip = theApp.glob_prefs->GetWebUseGzip();
	CString Out = "";
	CString OutE = "";	// List Entry Templates
	CString OutE2 = "";
	CString OutS = "";	// ServerStatus Templates
	TCHAR *gzipOut = NULL;
	long gzipLen;
	
	CString HTTPProcessData = "";
	CString HTTPTemp = "";
	char	HTTPTempC[100] = "";
	CString Password = theApp.glob_prefs->GetHTTPPass();

	long lSession = 0;
	if(_ParseURL(Data.sURL, "ses") != "")
		lSession = atol(_ParseURL(Data.sURL, "ses"));

	if (_ParseURL(Data.sURL, "w") == "password")
	{
		if(MD5Sum(_ParseURL(Data.sURL, "p")).GetHash() == Password)
		{
			Session ses;
			ses.startTime = CTime::GetCurrentTime();
			long l1 = rand(), l2 = rand();
			ses.lSession = lSession = l1 * 10000L + l2;
			pThis->m_Params.Sessions.Add(ses);
		}
		isUseGzip = false; // [Julien]
	}

	CString sSession; sSession.Format("%ld", lSession);

	if (_ParseURL(Data.sURL, "w") == "logout") 
		_RemoveSession(Data, lSession);

	if(_IsLoggedIn(Data, lSession))
	{
		Out += _GetHeader(Data, lSession);
		CString sPage = _ParseURL(Data.sURL, "w");
		if (sPage == "server") 
		{
			Out += _GetServerList(Data);
		}
		else
		if (sPage == "download") 
		{
			Out += _GetDownloadLink(Data);
		}
		else
		if (sPage == "shared") 
		{ 
			Out += _GetSharedFilesList(Data);
		}
		else
		if (sPage == "transfer") 
		{
			Out += _GetTransferList(Data);
		}
		else
		if (sPage == "websearch") 
		{
			Out += _GetWebSearch(Data);
		}
		else
		if (sPage == "graphs")
		{
			Out += _GetGraphs(Data);
		}
		else
		if (sPage == "log") 
		{
			Out += _GetLog(Data);
		}
		if (sPage == "sinfo") 
		{
			Out += _GetServerInfo(Data);
		}
		if (sPage == "debuglog") 
		{
			Out += _GetDebugLog(Data);
		}
		if (sPage == "stats") 
		{
			Out += _GetStats(Data);
		}
		if (sPage == "options") 
		{
			isUseGzip = false;
			Out += _GetPreferences(Data);
		}
		Out += _GetFooter(Data);

		if (sPage == "")
			isUseGzip = false;

		if(isUseGzip)
		{
			bool bOk = false;
			try
			{
				uLongf destLen = Out.GetLength() + 1024;
				gzipOut = new TCHAR[destLen];
				if(_GzipCompress((Bytef*)gzipOut, &destLen, (const Bytef*)(TCHAR*)Out.GetBuffer(0), Out.GetLength(), Z_DEFAULT_COMPRESSION) == Z_OK)
				{
					bOk = true;
					gzipLen = destLen;
				}
			}
			catch(...)
			{
			}
			if(!bOk)
			{
				isUseGzip = false;
				if(gzipOut != NULL)
				{
					delete[] gzipOut;
					gzipOut = NULL;
				}
			}
		}
	}
	else 
	{
		isUseGzip = false;
		Out += _GetLoginScreen(Data);
	}

	// send answer ...
	if(!isUseGzip)
	{
		Data.pSocket->SendContent(HTTPInit, Out, Out.GetLength());
	}
	else
	{
		Data.pSocket->SendContent(HTTPInitGZ, gzipOut, gzipLen);
	}
	if(gzipOut != NULL)
		delete[] gzipOut;

	EMULE_CATCH
}

CString CWebServer::_ParseURL(CString URL, CString fieldname)
{
	EMULE_TRY

	CString value = "";
	CString Parameter = "";
	char fromReplace[4] = "";	// decode URL
	char toReplace[2] = "";		// decode URL
	int i = 0;
	int findPos = -1;
	int findLength = 0;

	if (URL.Find("?") > -1) {
		Parameter = URL.Mid(URL.Find("?")+1, URL.GetLength()-URL.Find("?")-1);

		// search the fieldname beginning / middle and strip the rest...
		if (Parameter.Find(fieldname + "=") == 0) {
			findPos = 0;
			findLength = fieldname.GetLength() + 1;
		}
		if (Parameter.Find("&" + fieldname + "=") > -1) {
			findPos = Parameter.Find("&" + fieldname + "=");
			findLength = fieldname.GetLength() + 2;
		}
		if (findPos > -1) {
			Parameter = Parameter.Mid(findPos + findLength, Parameter.GetLength());
			if (Parameter.Find("&") > -1) {
				Parameter = Parameter.Mid(0, Parameter.Find("&"));
			}
	
			value = Parameter;

			// decode value ...
			value.Replace("+", " ");
			for (i = 0 ; i <= 255 ; i++) {
				sprintf(fromReplace, "%%%02x", i);
				toReplace[0] = (char)i;
				toReplace[1] = NULL;
				value.Replace(fromReplace, toReplace);
				sprintf(fromReplace, "%%%02X", i);
				value.Replace(fromReplace, toReplace);
			}
		}
	}

	return value;

	EMULE_CATCH

	return "";
}

CString CWebServer::_GetHeader(ThreadData Data, long lSession)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return "";

	CString sSession; sSession.Format("%ld", lSession);

	CString Out = pThis->m_Templates.sHeader;

	Out.Replace("[CharSet]", _GetWebCharSet());

	if(theApp.glob_prefs->GetWebPageRefresh() != 0)
	{
		CString sPage = _ParseURL(Data.sURL, "w");
		if ((sPage == "transfer") ||
			(sPage == "server") ||
			(sPage == "graphs") ||
			(sPage == "log") ||
			(sPage == "sinfo") ||
			(sPage == "debuglog") ||
			(sPage == "stats"))
		{
			CString sT = pThis->m_Templates.sHeaderMetaRefresh;
			CString sRefresh; sRefresh.Format("%d", theApp.glob_prefs->GetWebPageRefresh());
			sT.Replace("[RefreshVal]", sRefresh);
			sT.Replace("[wCommand]", _ParseURL(Data.sURL, "w"));
			Out.Replace("[HeaderMeta]", sT);
		}
	}
	Out.Replace("[Session]", sSession);
	Out.Replace("[HeaderMeta]", ""); // In case there are no meta
	Out.Replace("[eMulePlus]", _GetPlainResString(IDS_EMULE_PLUS));
	Out.Replace("[version]", CURRENT_VERSION_LONG);
	Out.Replace("[StyleSheet]", pThis->m_Templates.sHeaderStylesheet);
	Out.Replace("[WebControl]", _GetPlainResString(IDS_WEB_CONTROL));
	Out.Replace("[Transfer]", _GetPlainResString(IDS_CD_TRANS));
	Out.Replace("[Server]", _GetPlainResString(IDS_SV_SERVERLIST));
	Out.Replace("[Shared]", _GetPlainResString(IDS_SHAREDFILES));
	Out.Replace("[Download]", _GetPlainResString(IDS_SW_LINK));
	Out.Replace("[Graphs]", _GetPlainResString(IDS_GRAPHS));
	Out.Replace("[Log]", _GetPlainResString(IDS_SV_LOG));
	Out.Replace("[ServerInfo]", _GetPlainResString(IDS_SV_SERVERINFO));
	Out.Replace("[DebugLog]", _GetPlainResString(IDS_SV_DEBUGLOG));
	Out.Replace("[Stats]", _GetPlainResString(IDS_SF_STATISTICS));
	Out.Replace("[Options]", _GetPlainResString(IDS_EM_PREFS));
	Out.Replace("[Logout]", _GetPlainResString(IDS_WEB_LOGOUT));
	char HTTPTempC[100] = "";
	CString sConnected = "";
	if (theApp.serverconnect->IsConnected()) 
	{
		if(!theApp.serverconnect->IsLowID())
			sConnected = _GetPlainResString(IDS_CONNECTED);
		else
			sConnected = _GetPlainResString(IDS_CONNECTED) + " (" + _GetPlainResString(IDS_LOWID) + ")";

		CServer* cur_server = theApp.serverconnect->GetCurrentServer();
		sConnected += ": " + CString(cur_server->GetListName());

		sprintf(HTTPTempC, "%i ", pThis->m_Params.nUsers);
		sConnected += " [" + CString(HTTPTempC) + _GetPlainResString(IDS_LUSERS) + "]";

	} 
	else if (theApp.serverconnect->IsConnecting())
	{
		sConnected = _GetPlainResString(IDS_CONNECTING);
	}
	else
	{
		sConnected = _GetPlainResString(IDS_DISCONNECTED);
	}
	Out.Replace("[Connected]", sConnected);
    sprintf(HTTPTempC, _GetPlainResString(IDS_UPDOWN),(float)theApp.uploadqueue->GetDatarate()/1024,(float)theApp.downloadqueue->GetDatarate()/1024);
	CString sLimits;
	// EC 25-12-2003
	CString MaxUpload;
	CString MaxDownload;
	MaxUpload.Format("%i",theApp.glob_prefs->GetMaxUpload());
	MaxDownload.Format("%i",theApp.glob_prefs->GetMaxDownload());
	if (MaxUpload == "65535")  MaxUpload = GetResString(IDS_PW_UNLIMITED);
	if (MaxDownload == "65535") MaxDownload = GetResString(IDS_PW_UNLIMITED);
	sLimits.Format("%s/%s", MaxUpload, MaxDownload);
	// EC Ends
	Out.Replace("[Speed]", " ( " + CString(HTTPTempC) + " | " + _GetPlainResString(IDS_PW_CON_LIMIT) + ": " + sLimits + " )");

	return Out;

	EMULE_CATCH

	return "";
}

CString CWebServer::_GetFooter(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return "";

	return pThis->m_Templates.sFooter;

	EMULE_CATCH

	return "";
}

CString CWebServer::_GetServerList(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return "";

	CString sSession = _ParseURL(Data.sURL, "ses");

	CString sAddServerBox = "";

	CString sCmd = _ParseURL(Data.sURL, "c");
	if (sCmd == "connect")
	{
		CString sIP = _ParseURL(Data.sURL, "ip");
		if(sIP.IsEmpty())
			theApp.emuledlg->SendMessage(WM_COMMAND, MP_CONNECT);
		else
			_ConnectToServer(sIP);
	}	
	else if (sCmd == "disconnect") 
	{
		theApp.emuledlg->SendMessage(WM_COMMAND, MP_DISCONNECT);
	}	
	else if (sCmd == "remove") 
	{
		CString sIP = _ParseURL(Data.sURL, "ip");
		if(!sIP.IsEmpty())
			_RemoveServer(sIP);
	}	
	else if (sCmd == "options") 
	{
		sAddServerBox = _GetAddServerBox(Data);
	}
	CString sSort = _ParseURL(Data.sURL, "sort");
	if (sSort != "") 
	{
		if(sSort == "name")
			pThis->m_Params.ServerSort = SERVER_SORT_NAME;
		else
		if(sSort == "description")
			pThis->m_Params.ServerSort = SERVER_SORT_DESCRIPTION;
		else
		if(sSort == "ip")
			pThis->m_Params.ServerSort = SERVER_SORT_IP;
		else
		if(sSort == "users")
			pThis->m_Params.ServerSort = SERVER_SORT_USERS;
		else
		if(sSort == "files")
			pThis->m_Params.ServerSort = SERVER_SORT_FILES;

		if(_ParseURL(Data.sURL, "sortreverse") == "")
			pThis->m_Params.bServerSortReverse = false;
	}
	if (_ParseURL(Data.sURL, "sortreverse") != "") 
	{
		pThis->m_Params.bServerSortReverse = (_ParseURL(Data.sURL, "sortreverse") == "true");
	}
	CString sServerSortRev;
	if(pThis->m_Params.bServerSortReverse)
		sServerSortRev = "false";
	else
		sServerSortRev = "true";

	CString Out = pThis->m_Templates.sServerList;
	Out.Replace("[ConnectedServerData]", _GetConnectedServer(Data));
    Out.Replace("[AddServerBox]", sAddServerBox);
	Out.Replace("[Session]", sSession);
	if(pThis->m_Params.ServerSort == SERVER_SORT_NAME)
		Out.Replace("[SortName]", "&sortreverse=" + sServerSortRev);
	else
		Out.Replace("[SortName]", "");
	if(pThis->m_Params.ServerSort == SERVER_SORT_DESCRIPTION)
		Out.Replace("[SortDescription]", "&sortreverse=" + sServerSortRev);
	else
		Out.Replace("[SortDescription]", "");
	if(pThis->m_Params.ServerSort == SERVER_SORT_IP)
		Out.Replace("[SortIP]", "&sortreverse=" + sServerSortRev);
	else
		Out.Replace("[SortIP]", "");
	if(pThis->m_Params.ServerSort == SERVER_SORT_USERS)
		Out.Replace("[SortUsers]", "&sortreverse=" + sServerSortRev);
	else
		Out.Replace("[SortUsers]", "");
	if(pThis->m_Params.ServerSort == SERVER_SORT_FILES)
		Out.Replace("[SortFiles]", "&sortreverse=" + sServerSortRev);
	else
		Out.Replace("[SortFiles]", "");
	Out.Replace("[ServerList]", _GetPlainResString(IDS_SV_SERVERLIST));
	Out.Replace("[Servername]", _GetPlainResString(IDS_SL_SERVERNAME));
	Out.Replace("[Description]", _GetPlainResString(IDS_DESCRIPTION));
	Out.Replace("[Address]", _GetPlainResString(IDS_IP));
	Out.Replace("[Connect]", _GetPlainResString(IDS_IRC_CONNECT));
	Out.Replace("[Users]", _GetPlainResString(IDS_LUSERS));
	Out.Replace("[Files]", _GetPlainResString(IDS_LFILES));
	Out.Replace("[Actions]", _GetPlainResString(IDS_WEB_ACTIONS));

	CString OutE = pThis->m_Templates.sServerLine;
	OutE.Replace("[Connect]", _GetPlainResString(IDS_IRC_CONNECT));
	OutE.Replace("[RemoveServer]", _GetPlainResString(IDS_REMOVETHIS));
	OutE.Replace("[ConfirmRemove]", _GetPlainResString(IDS_WEB_CONFIRM_REMOVE_SERVER));

	CArray<ServerEntry, ServerEntry> ServerArray;

	// Populating array
	for (POSITION pos=theApp.serverlist->list.GetHeadPosition();pos != 0;theApp.serverlist->list.GetNext(pos))
	{
		CServer* cur_file =  theApp.serverlist->list.GetAt(pos);
		ServerEntry Entry;
		Entry.sServerName = _SpecialChars(cur_file->GetListName());
		Entry.sServerDescription = _SpecialChars(cur_file->GetDescription());
		Entry.nServerPort = cur_file->GetPort();
		Entry.sServerIP = cur_file->GetAddress();
		Entry.nServerUsers = cur_file->GetUsers();
		Entry.nServerMaxUsers = cur_file->GetMaxUsers();
		Entry.nServerFiles = cur_file->GetFiles();

		ServerArray.Add(Entry);
	}
	// Sorting (simple bubble sort, we don't have tons of data here)
	bool bSorted = true;
	for(int nMax = 0;bSorted && nMax < ServerArray.GetCount()*2; nMax++)
	{
		bSorted = false;
		for(int i = 0; i < ServerArray.GetCount() - 1; i++)
		{
			bool bSwap = false;
			switch(pThis->m_Params.ServerSort)
			{
			case SERVER_SORT_NAME:
				bSwap = ServerArray[i].sServerName.CompareNoCase(ServerArray[i+1].sServerName) > 0;
				break;
			case SERVER_SORT_DESCRIPTION:
				bSwap = ServerArray[i].sServerDescription.CompareNoCase(ServerArray[i+1].sServerDescription) < 0;
				break;
			case SERVER_SORT_IP:
				bSwap = ServerArray[i].sServerIP.CompareNoCase(ServerArray[i+1].sServerIP) > 0;
				break;
			case SERVER_SORT_USERS:
				bSwap = ServerArray[i].nServerUsers < ServerArray[i+1].nServerUsers;
				break;
			case SERVER_SORT_FILES:
				bSwap = ServerArray[i].nServerFiles < ServerArray[i+1].nServerFiles;
				break;
			}
			if(pThis->m_Params.bServerSortReverse)
			{
				bSwap = !bSwap;
			}
			if(bSwap)
			{
				bSorted = true;
				ServerEntry TmpEntry = ServerArray[i];
				ServerArray[i] = ServerArray[i+1];
				ServerArray[i+1] = TmpEntry;
			}
		}
	}
	// Displaying
	CString sList = "";
	for(int i = 0; i < ServerArray.GetCount(); i++)
	{
		CString HTTPProcessData = OutE;	// Copy Entry Line to Temp
		HTTPProcessData.Replace("[1]", ServerArray[i].sServerName);
		HTTPProcessData.Replace("[2]", ServerArray[i].sServerDescription);
		CString sPort; sPort.Format(":%d", ServerArray[i].nServerPort);
		HTTPProcessData.Replace("[3]", ServerArray[i].sServerIP + sPort);
		CString sT;
		if(ServerArray[i].nServerUsers > 0)
		{
			if(ServerArray[i].nServerMaxUsers > 0)
				sT.Format("%d (%d)", ServerArray[i].nServerUsers, ServerArray[i].nServerMaxUsers);
			else
				sT.Format("%d", ServerArray[i].nServerUsers);
		}
		HTTPProcessData.Replace("[4]", sT);
		sT = "";
		if(ServerArray[i].nServerFiles > 0)
			sT.Format("%d", ServerArray[i].nServerFiles);
		HTTPProcessData.Replace("[5]", sT);
		HTTPProcessData.Replace("[6]", "/?ses=" + sSession + "&w=server&c=connect&ip=" + ServerArray[i].sServerIP);
		HTTPProcessData.Replace("[LinkRemove]", "/?ses=" + sSession + "&w=server&c=remove&ip=" + ServerArray[i].sServerIP);

		sList += HTTPProcessData;
	}
	Out.Replace("[ServersList]", sList);

	return Out;

	EMULE_CATCH

	return "";
}

CString CWebServer::_GetTransferList(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return "";

	CString sSession = _ParseURL(Data.sURL, "ses");

	CString Out = "";
	if (_ParseURL(Data.sURL, "c") != "") 
	{
		CString HTTPTemp = _ParseURL(Data.sURL, "c");
		if (HTTPTemp.Right(1) != "/")
			HTTPTemp += "/";
		try 
		{
			CED2KLink* pLink = CED2KLink::CreateLinkFromUrl(HTTPTemp);
			_ASSERT( pLink !=0 );
			if (pLink->GetKind() == CED2KLink::kFile)
				theApp.downloadqueue->AddFileLinkToDownload(pLink->GetFileLink());
			else
				throw CString("bad link");
			delete pLink;
		}
		catch(CString error)
		{
			char HTTPTempC[100] = "";
			sprintf(HTTPTempC,_GetPlainResString(IDS_ERR_INVALIDLINK),error.GetBuffer());
			Out += pThis->m_Templates.sTransferBadLink;
			Out.Replace("[InvalidLink]", HTTPTempC);
			Out.Replace("[Link]", HTTPTemp);
		}
	}
	if (_ParseURL(Data.sURL, "op") != "" &&
		_ParseURL(Data.sURL, "file") != "")
	{
		uchar FileHash[16];
		_GetFileHash(_ParseURL(Data.sURL, "file"), FileHash);

		CPartFile* found_file = theApp.downloadqueue->GetFileByID(FileHash);
		
		if(_ParseURL(Data.sURL, "op") == "pause")
		{
			if(found_file)
				found_file->PauseFile();
		}
		else if(_ParseURL(Data.sURL, "op") == "resume")
		{
			if(found_file)
				found_file->ResumeFile();
		}
		else if(_ParseURL(Data.sURL, "op") == "cancel")
		{
			if(found_file)
				found_file->DeleteFile();
		}
	}
	if (_ParseURL(Data.sURL, "sort") != "") 
	{
		if(_ParseURL(Data.sURL, "sort") == "name")
			pThis->m_Params.DownloadSort = DOWN_SORT_NAME;
		else
		if(_ParseURL(Data.sURL, "sort") == "size")
			pThis->m_Params.DownloadSort = DOWN_SORT_SIZE;
		else
		if(_ParseURL(Data.sURL, "sort") == "transferred")
			pThis->m_Params.DownloadSort = DOWN_SORT_TRANSFERRED;
		else
		if(_ParseURL(Data.sURL, "sort") == "speed")
			pThis->m_Params.DownloadSort = DOWN_SORT_SPEED;
		else
		if(_ParseURL(Data.sURL, "sort") == "progress")
			pThis->m_Params.DownloadSort = DOWN_SORT_PROGRESS;

		if(_ParseURL(Data.sURL, "sortreverse") == "")
			pThis->m_Params.bDownloadSortReverse = false;
	}
	if (_ParseURL(Data.sURL, "sortreverse") != "") 
	{
		pThis->m_Params.bDownloadSortReverse = (_ParseURL(Data.sURL, "sortreverse") == "true");
	}
	if(_ParseURL(Data.sURL, "showuploadqueue") == "true")
	{
		pThis->m_Params.bShowUploadQueue = true;
	}
	if(_ParseURL(Data.sURL, "showuploadqueue") == "false")
	{
		pThis->m_Params.bShowUploadQueue = false;
	}
	CString sDownloadSortRev;
	if(pThis->m_Params.bDownloadSortReverse)
		sDownloadSortRev = "false";
	else
		sDownloadSortRev = "true";

	Out += pThis->m_Templates.sTransferImages;
	Out += pThis->m_Templates.sTransferList;
	Out.Replace("[DownloadHeader]", pThis->m_Templates.sTransferDownHeader);
	Out.Replace("[DownloadFooter]", pThis->m_Templates.sTransferDownFooter);
	Out.Replace("[UploadHeader]", pThis->m_Templates.sTransferUpHeader);
	Out.Replace("[UploadFooter]", pThis->m_Templates.sTransferUpFooter);
	Out.Replace("[Session]", sSession);
	if(pThis->m_Params.DownloadSort == DOWN_SORT_NAME)
		Out.Replace("[SortName]", "&sortreverse=" + sDownloadSortRev);
	else
		Out.Replace("[SortName]", "");
	if(pThis->m_Params.DownloadSort == DOWN_SORT_SIZE)
		Out.Replace("[SortSize]", "&sortreverse=" + sDownloadSortRev);
	else
		Out.Replace("[SortSize]", "");
	if(pThis->m_Params.DownloadSort == DOWN_SORT_TRANSFERRED)
		Out.Replace("[SortTransferred]", "&sortreverse=" + sDownloadSortRev);
	else
		Out.Replace("[SortTransferred]", "");
	if(pThis->m_Params.DownloadSort == DOWN_SORT_SPEED)
		Out.Replace("[SortSpeed]", "&sortreverse=" + sDownloadSortRev);
	else
		Out.Replace("[SortSpeed]", "");
	if(pThis->m_Params.DownloadSort == DOWN_SORT_PROGRESS)
		Out.Replace("[SortProgress]", "&sortreverse=" + sDownloadSortRev);
	else
		Out.Replace("[SortProgress]", "");
	Out.Replace("[DownloadList]", _GetPlainResString(IDS_ST_ACTIVEDOWNLOAD));
	Out.Replace("[Filename]", _GetPlainResString(IDS_DL_FILENAME));
	Out.Replace("[Size]", _GetPlainResString(IDS_DL_SIZE));
	Out.Replace("[Transferred]", _GetPlainResString(IDS_DL_TRANSF));
	Out.Replace("[Percent]", _GetPlainResString(IDS_SF_COMPLETED));
	Out.Replace("[Speed]", _GetPlainResString(IDS_DL_SPEED));
	Out.Replace("[Sources]", _GetPlainResString(IDS_DL_SOURCES));
	Out.Replace("[Actions]", _GetPlainResString(IDS_WEB_ACTIONS));
	Out.Replace("[UploadList]", _GetPlainResString(IDS_ST_ACTIVEUPLOAD));
	Out.Replace("[User]", _GetPlainResString(IDS_QL_USERNAME));
	Out.Replace("[TotalDown]", _GetPlainResString(IDS_INFLST_USER_TOTALDOWNLOAD));
	Out.Replace("[TotalUp]", _GetPlainResString(IDS_INFLST_USER_TOTALUPLOAD));

	CString OutE = pThis->m_Templates.sTransferDownLine;
	CString OutE2 = pThis->m_Templates.sTransferDownLineGood;

	float fTotalSize = 0, fTotalTransferred = 0, fTotalSpeed = 0;
	CArray<DownloadFiles, DownloadFiles> FilesArray;
	// Populating array
	for (POSITION pos=theApp.downloadqueue->filelist.GetHeadPosition();pos != 0;theApp.downloadqueue->filelist.GetNext(pos))
	{
		CPartFile* cur_file =  theApp.downloadqueue->filelist.GetAt(pos);

		DownloadFiles dFile;
		dFile.sFileName = _SpecialChars(cur_file->GetFileName());
		dFile.lFileSize = cur_file->GetFileSize();
		dFile.lFileTransferred = cur_file->GetTransfered();
		dFile.lFileSpeed = cur_file->GetDatarate();
		dFile.nFileStatus = cur_file->GetStatus();
		dFile.sFileStatus = cur_file->GetPartfileStatus();
		dFile.sFileHash = _GetStringHash(cur_file->GetFileHash());
		dFile.lSourceCount = cur_file->GetSourceCount();
		dFile.lNotCurrentSourceCount = cur_file->GetNotCurrentSourcesCount();
		dFile.lTransferringSourceCount = cur_file->GetTransferingSrcCount();
		if (theApp.serverconnect->IsConnected() && !theApp.serverconnect->IsLowID())
			dFile.sED2kLink = theApp.CreateED2kSourceLink(cur_file);
		else
            dFile.sED2kLink = theApp.CreateED2kLink(cur_file);
		dFile.sFileInfo = _SpecialChars(theApp.emuledlg->transferwnd.GetDownloadFileInfo(cur_file));

		FilesArray.Add(dFile);
	}

	// Sorting (simple bubble sort, we don't have tons of data here)
	bool bSorted = true;
	for(int nMax = 0;bSorted && nMax < FilesArray.GetCount()*2; nMax++)
	{
		bSorted = false;
		for(int i = 0; i < FilesArray.GetCount() - 1; i++)
		{
			bool bSwap = false;
			switch(pThis->m_Params.DownloadSort)
			{
			case DOWN_SORT_NAME:
				bSwap = FilesArray[i].sFileName.CompareNoCase(FilesArray[i+1].sFileName) > 0;
				break;
			case DOWN_SORT_SIZE:
				bSwap = FilesArray[i].lFileSize < FilesArray[i+1].lFileSize;
				break;
			case DOWN_SORT_TRANSFERRED:
				bSwap = FilesArray[i].lFileTransferred < FilesArray[i+1].lFileTransferred;
				break;
			case DOWN_SORT_SPEED:
				bSwap = FilesArray[i].lFileSpeed < FilesArray[i+1].lFileSpeed;
				break;
			case DOWN_SORT_PROGRESS:
				{
					int nPercent1 = 0, nPercent2 = 0;
					if(FilesArray[i].lFileSize > 0)
					{
						float fS = FilesArray[i].lFileSize, fT = FilesArray[i].lFileTransferred;
						nPercent1 = abs((fT * 100.0) / fS);
					}
					if(FilesArray[i+1].lFileSize > 0)
					{
						float fS = FilesArray[i+1].lFileSize, fT = FilesArray[i+1].lFileTransferred;
						nPercent2 = abs((fT * 100.0) / fS);
					}
					bSwap = nPercent1 < nPercent2;
				}
			}
			if(pThis->m_Params.bDownloadSortReverse)
			{
				bSwap = !bSwap;
			}
			if(bSwap)
			{
				bSorted = true;
				DownloadFiles TmpFile = FilesArray[i];
				FilesArray[i] = FilesArray[i+1];
				FilesArray[i+1] = TmpFile;
			}
		}
	}

	// Displaying
	CString sDownList = "";
	for(int i = 0; i < FilesArray.GetCount(); i++)
	{
		CString sActions = "<acronym title=\"" + FilesArray[i].sFileStatus + "\"><a href=\"javascript:alert(\'Here will be file info and options like rename\')\"><img src=\"javascript:img_info\" alt=\"" + FilesArray[i].sFileStatus + "\"></a></acronym> ";
		
		CString sED2kLink;
		sED2kLink.Format("<acronym title=\"[Ed2klink]\"><a href=\""+ FilesArray[i].sED2kLink +"\"><img src=\"javascript:img_link\" alt=\"[Ed2klink]\"></a></acronym> ");
		sED2kLink.Replace("[Ed2klink]", _GetPlainResString(IDS_SW_LINK));
		sActions += sED2kLink;

		bool bCanBeDeleted = true;
		switch(FilesArray[i].nFileStatus)
		{
		case PS_COMPLETING:
		case PS_COMPLETE:
			bCanBeDeleted = false;
			break;
		case PS_HASHING: 
		case PS_WAITINGFORHASH:
		case PS_ERROR:
			break;
		case PS_PAUSED:
		{
			CString sResume;
			sResume.Format("<acronym title=\"[Resume]\"><a href=\"[Link]\"><img src=\"javascript:img_resume\" alt=\"[Resume]\"></a></acronym> ");
			sResume.Replace("[Link]", "/?ses=" + sSession + "&w=transfer&op=resume&file=" + FilesArray[i].sFileHash);
			sActions += sResume;
		}
			break; 
		default: // waiting or downloading
		{
			CString sPause;
			sPause.Format("<acronym title=\"[Pause]\"><a href=\"[Link]\"><img src=\"javascript:img_pause\" alt=\"[Pause]\"></a></acronym> ");
			sPause.Replace("[Link]", "/?ses=" + sSession + "&w=transfer&op=pause&file=" + FilesArray[i].sFileHash);
			sActions += sPause;
		}
			break;
		}
		if(bCanBeDeleted)
		{
			CString sCancel;
			sCancel.Format("<acronym title=\"[Cancel]\"><a href=\"[Link]\" onclick=\"return confirm(\'[ConfirmCancel]\')\"><img src=\"javascript:img_cancel\" alt=\"[Cancel]\"></a></acronym> ");
			sCancel.Replace("[Link]", "/?ses=" + sSession + "&w=transfer&op=cancel&file=" + FilesArray[i].sFileHash);
			sActions += sCancel;
		}
		sActions.Replace("[Resume]", _GetPlainResString(IDS_DL_RESUME));
		sActions.Replace("[Pause]", _GetPlainResString(IDS_DL_PAUSE));
		sActions.Replace("[Cancel]", _GetPlainResString(IDS_MAIN_BTN_CANCEL));
		sActions.Replace("[ConfirmCancel]", _GetPlainResString(IDS_Q_CANCELDL2, true));

		CString HTTPProcessData;
		// if downloading, draw in other color
		if(FilesArray[i].lFileSpeed > 0)
			HTTPProcessData = OutE2;
		else
			HTTPProcessData = OutE;

		if(FilesArray[i].sFileName.GetLength() > SHORT_FILENAME_LENGTH)
		HTTPProcessData.Replace("[ShortFileName]", FilesArray[i].sFileName.Left(SHORT_FILENAME_LENGTH) + "...");
		else
			HTTPProcessData.Replace("[ShortFileName]", FilesArray[i].sFileName);
		HTTPProcessData.Replace("[FileInfo]", FilesArray[i].sFileInfo);

		fTotalSize += FilesArray[i].lFileSize;

		char HTTPTempC[100] = "";
		sprintf(HTTPTempC, "%8.2f ", FilesArray[i].lFileSize/1048576.0);
		CString HTTPTemp = HTTPTempC + _GetPlainResString(IDS_MBYTES);
		HTTPProcessData.Replace("[2]", HTTPTemp);

		if(FilesArray[i].lFileTransferred > 0)
		{
			fTotalTransferred += FilesArray[i].lFileTransferred;

			sprintf(HTTPTempC, "%8.2f ", FilesArray[i].lFileTransferred/1048576.0);
			HTTPTemp = HTTPTempC + _GetPlainResString(IDS_MBYTES);												
			HTTPProcessData.Replace("[3]", HTTPTemp);
		}
		else
			HTTPProcessData.Replace("[3]", "-");

		int nPercent = 0;
		if(FilesArray[i].lFileSize > 0)
		{
			float fS = FilesArray[i].lFileSize, fT = FilesArray[i].lFileTransferred;
			nPercent = abs((fT * 100.0) / fS);
		}
		sprintf(HTTPTempC, "%d", nPercent);
		HTTPProcessData.Replace("[Percent]", CString(HTTPTempC));

		if(FilesArray[i].lFileSpeed > 0.0f)
		{
			fTotalSpeed += FilesArray[i].lFileSpeed;

			sprintf(HTTPTempC, "%8.2f ", FilesArray[i].lFileSpeed/1024.0);
			HTTPTemp = HTTPTempC + _GetPlainResString(IDS_KBYTESEC);												
			HTTPProcessData.Replace("[4]", HTTPTemp);
		}
		else
			HTTPProcessData.Replace("[4]", "-");
		if(FilesArray[i].lSourceCount > 0)
		{
		sprintf(HTTPTempC, "%8i/%8i (%i)",
			FilesArray[i].lSourceCount-FilesArray[i].lNotCurrentSourceCount,
			FilesArray[i].lSourceCount,
			FilesArray[i].lTransferringSourceCount
		);
		HTTPTemp = HTTPTempC;
		HTTPProcessData.Replace("[5]", HTTPTemp);
		}
		else
			HTTPProcessData.Replace("[5]", "-");

		HTTPProcessData.Replace("[6]", sActions);

		sDownList += HTTPProcessData;
	}

	Out.Replace("[DownloadFilesList]", sDownList);
	CString sT;
	sT.Format("%8.0f ", fTotalSize/1048576.0);
	sT += _GetPlainResString(IDS_MBYTES);
	Out.Replace("[TotalDownSize]", sT);
	sT.Format("%8.0f ", fTotalTransferred/1048576.0);
	sT += _GetPlainResString(IDS_MBYTES);
	Out.Replace("[TotalDownTransferred]", sT);
	sT.Format("%8.2f ", fTotalSpeed/1024.0);
	sT += _GetPlainResString(IDS_KBYTESEC);
	Out.Replace("[TotalDownSpeed]", sT);
	OutE = pThis->m_Templates.sTransferUpLine;

	fTotalSize = 0;
	fTotalTransferred = 0;
	fTotalSpeed = 0;

	CString sUpList = "";
	for (POSITION pos = theApp.uploadqueue->uploadinglist.GetHeadPosition(); pos != 0;theApp.uploadqueue->uploadinglist.GetNext(pos))
	{
		CUpDownClient* cur_client = theApp.uploadqueue->uploadinglist.GetAt(pos);
		CString HTTPProcessData = OutE;
		HTTPProcessData.Replace("[1]", _SpecialChars(cur_client->GetUserName()));
		HTTPProcessData.Replace("[FileInfo]", _SpecialChars(theApp.emuledlg->transferwnd.GetUploadFileInfo(cur_client)));

		CKnownFile* file = theApp.sharedfiles->GetFileByID(cur_client->reqfileid);
		if (file)
			HTTPProcessData.Replace("[2]", _SpecialChars(CString(file->GetFileName())));
		else
			HTTPProcessData.Replace("[2]", _GetPlainResString(IDS_REQ_UNKNOWNFILE));

		fTotalSize += cur_client->GetTransferedDown();
		fTotalTransferred += cur_client->GetTransferedUp();
		CString HTTPTemp;
		HTTPTemp.Format("%8.2f " + _GetPlainResString(IDS_MBYTES) + " / %8.2f " + _GetPlainResString(IDS_MBYTES), 
			cur_client->GetTransferedDown()/1048576.0, cur_client->GetTransferedUp()/1048576.0);
		HTTPProcessData.Replace("[3]", HTTPTemp);

		fTotalSpeed += cur_client->GetDatarate();
		HTTPTemp.Format("%8.2f " + _GetPlainResString(IDS_KBYTESEC), cur_client->GetDatarate()/1024.0);
		HTTPProcessData.Replace("[4]", HTTPTemp);
		sUpList += HTTPProcessData;
	}
	Out.Replace("[UploadFilesList]", sUpList);
	sT.Format("%8.2f " + _GetPlainResString(IDS_MBYTES) + " / %8.2f " + _GetPlainResString(IDS_MBYTES),
		fTotalSize/1048576.0, fTotalTransferred/1048576.0);
	Out.Replace("[TotalUpTransferred]", sT);
	sT.Format("%8.2f " + _GetPlainResString(IDS_KBYTESEC), fTotalSpeed/1024.0);
	Out.Replace("[TotalUpSpeed]", sT);

	if(pThis->m_Params.bShowUploadQueue) 
	{
		Out.Replace("[UploadQueue]", pThis->m_Templates.sTransferUpQueueShow);
		Out.Replace("[UploadQueueList]", _GetPlainResString(IDS_ONQUEUE));
		Out.Replace("[UserNameTitle]", _GetPlainResString(IDS_QL_USERNAME));
		Out.Replace("[FileNameTitle]", _GetPlainResString(IDS_DL_FILENAME));
		Out.Replace("[ScoreTitle]", _GetPlainResString(IDS_SCORE));
		Out.Replace("[BannedTitle]", _GetPlainResString(IDS_BANNED));

		OutE = pThis->m_Templates.sTransferUpQueueLine;
		// Replace [xx]
		CString sQueue = "";
		for (POSITION pos = theApp.uploadqueue->waitinglist.GetHeadPosition(); pos != 0;theApp.uploadqueue->waitinglist.GetNext(pos)){
			CUpDownClient* cur_client = theApp.uploadqueue->waitinglist.GetAt(pos);
			CString HTTPProcessData;
            char HTTPTempC[100] = "";
			HTTPProcessData = OutE;
			HTTPProcessData.Replace("[UserName]", _SpecialChars(cur_client->GetUserName()));
			if (cur_client->reqfile)
				HTTPProcessData.Replace("[FileName]", _SpecialChars(cur_client->reqfile->GetFileName()));
			else
				HTTPProcessData.Replace("[FileName]", "?");
			sprintf(HTTPTempC, "%i" , cur_client->GetScore(false));
			CString HTTPTemp = HTTPTempC;
			HTTPProcessData.Replace("[Score]", HTTPTemp);
			if (cur_client->IsBanned())
				HTTPProcessData.Replace("[Banned]", _GetPlainResString(IDS_YES));
			else
				HTTPProcessData.Replace("[Banned]", _GetPlainResString(IDS_NO));
			sQueue += HTTPProcessData;
		}
		Out.Replace("[QueueList]", sQueue);
	}
	else {
		Out.Replace("[UploadQueue]", pThis->m_Templates.sTransferUpQueueHide);
	}
	Out.Replace("[ShowQueue]", _GetPlainResString(IDS_WEB_SHOW_UPLOAD_QUEUE));
	Out.Replace("[HideQueue]", _GetPlainResString(IDS_WEB_HIDE_UPLOAD_QUEUE));
	Out.Replace("[Session]", sSession);

	return Out;

	EMULE_CATCH

	return "";
}

CString CWebServer::_GetDownloadLink(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return "";

	CString sSession = _ParseURL(Data.sURL, "ses");

	CString Out = pThis->m_Templates.sDownloadLink;

	Out.Replace("[Download]", _GetPlainResString(IDS_SW_DOWNLOAD));
	Out.Replace("[Ed2klink]", _GetPlainResString(IDS_SW_LINK));
	Out.Replace("[Start]", _GetPlainResString(IDS_SW_START));
	Out.Replace("[Session]", sSession);

	return Out;

	EMULE_CATCH

	return "";
}

CString CWebServer::_GetSharedFilesList(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
        CString sSession = _ParseURL(Data.sURL, "ses");
	if(pThis == NULL)
		return "";
	if (_ParseURL(Data.sURL, "sort") != "") 
	{
		if(_ParseURL(Data.sURL, "sort") == "name")
			pThis->m_Params.SharedSort = SHARED_SORT_NAME;
		else
		if(_ParseURL(Data.sURL, "sort") == "size")
			pThis->m_Params.SharedSort = SHARED_SORT_SIZE;
		else
		if(_ParseURL(Data.sURL, "sort") == "transferred")
			pThis->m_Params.SharedSort = SHARED_SORT_TRANSFERRED;
		else
		if(_ParseURL(Data.sURL, "sort") == "alltimetransferred")
			pThis->m_Params.SharedSort = SHARED_SORT_ALL_TIME_TRANSFERRED;
		else
		if(_ParseURL(Data.sURL, "sort") == "requests")
			pThis->m_Params.SharedSort = SHARED_SORT_REQUESTS;
		else
		if(_ParseURL(Data.sURL, "sort") == "alltimerequests")
			pThis->m_Params.SharedSort = SHARED_SORT_ALL_TIME_REQUESTS;
		else
		if(_ParseURL(Data.sURL, "sort") == "accepts")
			pThis->m_Params.SharedSort = SHARED_SORT_ACCEPTS;
		else
		if(_ParseURL(Data.sURL, "sort") == "alltimeaccepts")
			pThis->m_Params.SharedSort = SHARED_SORT_ALL_TIME_ACCEPTS;
		else
		if(_ParseURL(Data.sURL, "sort") == "priority")
			pThis->m_Params.SharedSort = SHARED_SORT_PRIORITY;

		if(_ParseURL(Data.sURL, "sortreverse") == "")
			pThis->m_Params.bSharedSortReverse = false;
	}
	if (_ParseURL(Data.sURL, "sortreverse") != "") 
		pThis->m_Params.bSharedSortReverse = (_ParseURL(Data.sURL, "sortreverse") == "true");

	if(_ParseURL(Data.sURL, "hash") != "" && _ParseURL(Data.sURL, "setpriority") != "") 
		_SetSharedFilePriority(_ParseURL(Data.sURL, "hash"),atoi(_ParseURL(Data.sURL, "setpriority")));

	if(_ParseURL(Data.sURL, "reload") == "true")
	{
		theApp.emuledlg->SendMessage(WEB_SHARED_FILES_RELOAD);
	}

	CString sSharedSortRev;
	if(pThis->m_Params.bSharedSortReverse)
		sSharedSortRev = "false";
	else
		sSharedSortRev = "true";
    //Name sorting link
	CString Out = pThis->m_Templates.sSharedList;
	if(pThis->m_Params.SharedSort == SHARED_SORT_NAME)
		Out.Replace("[SortName]", "sort=name&sortreverse=" + sSharedSortRev);
	else
		Out.Replace("[SortName]", "sort=name");
	//Size sorting Link
    if(pThis->m_Params.SharedSort == SHARED_SORT_SIZE)
		Out.Replace("[SortSize]", "sort=size&sortreverse=" + sSharedSortRev);
	else
		Out.Replace("[SortSize]", "sort=size");
	//Priority sorting Link
    if(pThis->m_Params.SharedSort == SHARED_SORT_PRIORITY)
		Out.Replace("[SortPriority]", "sort=priority&sortreverse=" + sSharedSortRev);
	else
		Out.Replace("[SortPriority]", "sort=priority");
    //Transferred sorting link
	if(pThis->m_Params.SharedSort == SHARED_SORT_TRANSFERRED)
	{
		if(pThis->m_Params.bSharedSortReverse)
            Out.Replace("[SortTransferred]", "sort=alltimetransferred&sortreverse=" + sSharedSortRev);
		else
			Out.Replace("[SortTransferred]", "sort=transferred&sortreverse=" + sSharedSortRev);
	}
	else
	if(pThis->m_Params.SharedSort == SHARED_SORT_ALL_TIME_TRANSFERRED)
	{
		if(pThis->m_Params.bSharedSortReverse)
            Out.Replace("[SortTransferred]", "sort=transferred&sortreverse=" + sSharedSortRev);
		else
			Out.Replace("[SortTransferred]", "sort=alltimetransferred&sortreverse=" + sSharedSortRev);
	}
	else
        Out.Replace("[SortTransferred]", "&sort=transferred&sortreverse=false");
    //Request sorting link
	if(pThis->m_Params.SharedSort == SHARED_SORT_REQUESTS)
	{
		if(pThis->m_Params.bSharedSortReverse)
            Out.Replace("[SortRequests]", "sort=alltimerequests&sortreverse=" + sSharedSortRev);
		else
			Out.Replace("[SortRequests]", "sort=requests&sortreverse=" + sSharedSortRev);
	}
	else
	if(pThis->m_Params.SharedSort == SHARED_SORT_ALL_TIME_REQUESTS)
	{
		if(pThis->m_Params.bSharedSortReverse)
            Out.Replace("[SortRequests]", "sort=requests&sortreverse=" + sSharedSortRev);
		else
			Out.Replace("[SortRequests]", "sort=alltimerequests&sortreverse=" + sSharedSortRev);
	}
	else
        Out.Replace("[SortRequests]", "&sort=requests&sortreverse=false");
    //Accepts sorting link
	if(pThis->m_Params.SharedSort == SHARED_SORT_ACCEPTS)
	{
		if(pThis->m_Params.bSharedSortReverse)
            Out.Replace("[SortAccepts]", "sort=alltimeaccepts&sortreverse=" + sSharedSortRev);
		else
			Out.Replace("[SortAccepts]", "sort=accepts&sortreverse=" + sSharedSortRev);
	}
	else
	if(pThis->m_Params.SharedSort == SHARED_SORT_ALL_TIME_ACCEPTS)
	{
		if(pThis->m_Params.bSharedSortReverse)
            Out.Replace("[SortAccepts]", "sort=accepts&sortreverse=" + sSharedSortRev);
		else
			Out.Replace("[SortAccepts]", "sort=alltimeaccepts&sortreverse=" + sSharedSortRev);
	}
	else
        Out.Replace("[SortAccepts]", "&sort=accepts&sortreverse=false");

	if(_ParseURL(Data.sURL, "reload") == "true")
	{
		CString resultlog = _SpecialChars(theApp.emuledlg->logtext);	//Pick-up last line of the log
		resultlog = resultlog.TrimRight('\n');
		resultlog = resultlog.Mid(resultlog.ReverseFind('\n'));
		Out.Replace("[Message]",resultlog);
	}
	else
        Out.Replace("[Message]","");

	Out.Replace("[Filename]", _GetPlainResString(IDS_DL_FILENAME));
	Out.Replace("[Priority]",  _GetPlainResString(IDS_PRIORITY));
    Out.Replace("[FileTransferred]",  _GetPlainResString(IDS_SF_TRANSFERRED));
    Out.Replace("[FileRequests]",  _GetPlainResString(IDS_SF_REQUESTS));
    Out.Replace("[FileAccepts]",  _GetPlainResString(IDS_SF_ACCEPTS));
	Out.Replace("[Size]", _GetPlainResString(IDS_DL_SIZE));
	Out.Replace("[Ed2klink]", _GetPlainResString(IDS_SW_LINK));
	Out.Replace("[Reload]", _GetPlainResString(IDS_SF_RELOAD));
	Out.Replace("[Session]", sSession);

	CString OutE = pThis->m_Templates.sSharedLine; 
	OutE.Replace("[Ed2klink]", _GetPlainResString(IDS_SW_LINK));
    OutE.Replace("[PriorityUp]", _GetPlainResString(IDS_PRIORITY_UP));
    OutE.Replace("[PriorityDown]", _GetPlainResString(IDS_PRIORITY_DOWN));

	CString OutE2 = pThis->m_Templates.sSharedLineChanged; 
	OutE.Replace("[Ed2klink]", _GetPlainResString(IDS_SW_LINK));
    OutE.Replace("[PriorityUp]", _GetPlainResString(IDS_PRIORITY_UP));
    OutE.Replace("[PriorityUp]", _GetPlainResString(IDS_PRIORITY_DOWN));

	CArray<SharedFiles, SharedFiles> SharedArray;
	// Populating array
	for (POSITION pos = theApp.sharedfiles->m_Files_map.GetStartPosition();pos != 0;)
	{
		CCKey bufKey;
		CKnownFile* cur_file;
		theApp.sharedfiles->m_Files_map.GetNextAssoc(pos,bufKey,cur_file);

		SharedFiles dFile;
		dFile.sFileName = _SpecialChars(cur_file->GetFileName());
		dFile.lFileSize = cur_file->GetFileSize();
		if (theApp.serverconnect->IsConnected()) 
			dFile.sED2kLink = theApp.CreateED2kSourceLink(cur_file);
		else
            dFile.sED2kLink = theApp.CreateED2kLink(cur_file);

		dFile.nFileTransferred = cur_file->statistic.GetTransfered();
		dFile.nFileAllTimeTransferred = cur_file->statistic.GetAllTimeTransfered();
		dFile.nFileRequests = cur_file->statistic.GetRequests();
		dFile.nFileAllTimeRequests = cur_file->statistic.GetAllTimeRequests();
		dFile.nFileAccepts = cur_file->statistic.GetAccepts();
		dFile.nFileAllTimeAccepts = cur_file->statistic.GetAllTimeAccepts();

		dFile.sFileHash = _GetStringHash(cur_file->GetFileHash());
		if (cur_file->IsAutoPrioritized())
		{
			if (cur_file->GetPriority() == PR_NORMAL)
				dFile.sFilePriority = _GetPlainResString(IDS_PRIOAUTONORMAL);
			else if (cur_file->GetPriority() == PR_HIGH)
				dFile.sFilePriority = _GetPlainResString(IDS_PRIOAUTOHIGH);
			else if (cur_file->GetPriority() == PR_VERYHIGH)
				dFile.sFilePriority = _GetPlainResString(IDS_PRIOAUTORELEASE);
		}
		else
		{
			if (cur_file->GetPriority() == PR_VERYLOW)
				dFile.sFilePriority = _GetPlainResString(IDS_PRIOVERYLOW);
			else if (cur_file->GetPriority() == PR_LOW)
				dFile.sFilePriority = _GetPlainResString(IDS_PRIOLOW);
			else if (cur_file->GetPriority() == PR_NORMAL)
				dFile.sFilePriority = _GetPlainResString(IDS_PRIONORMAL);
			else if (cur_file->GetPriority() == PR_HIGH)
				dFile.sFilePriority = _GetPlainResString(IDS_PRIOHIGH);
			else if (cur_file->GetPriority() == PR_VERYHIGH)
				dFile.sFilePriority = _GetPlainResString(IDS_PRIORELEASE);
		}
		dFile.nFilePriority = cur_file->GetPriority();
		dFile.bFileAutoPriority = cur_file->IsAutoPrioritized();
		SharedArray.Add(dFile);
	}

	// Sorting (simple bubble sort, we don't have tons of data here)
	bool bSorted = true;
	
	for(int nMax = 0;bSorted && nMax < SharedArray.GetCount()*2; nMax++)
	{
		bSorted = false;
		for(int i = 0; i < SharedArray.GetCount() - 1; i++)
		{
			bool bSwap = false;
			switch(pThis->m_Params.SharedSort)
			{
			case SHARED_SORT_NAME:
				bSwap = SharedArray[i].sFileName.CompareNoCase(SharedArray[i+1].sFileName) > 0;
				break;
			case SHARED_SORT_SIZE:
				bSwap = SharedArray[i].lFileSize < SharedArray[i+1].lFileSize;
				break;
			case SHARED_SORT_TRANSFERRED:
				bSwap = SharedArray[i].nFileTransferred < SharedArray[i+1].nFileTransferred;
				break;
			case SHARED_SORT_ALL_TIME_TRANSFERRED:
				bSwap = SharedArray[i].nFileAllTimeTransferred < SharedArray[i+1].nFileAllTimeTransferred;
				break;
			case SHARED_SORT_REQUESTS:
				bSwap = SharedArray[i].nFileRequests < SharedArray[i+1].nFileRequests;
				break;
			case SHARED_SORT_ALL_TIME_REQUESTS:
				bSwap = SharedArray[i].nFileAllTimeRequests < SharedArray[i+1].nFileAllTimeRequests;
				break;
			case SHARED_SORT_ACCEPTS:
				bSwap = SharedArray[i].nFileAccepts < SharedArray[i+1].nFileAccepts;
				break;
			case SHARED_SORT_ALL_TIME_ACCEPTS:
				bSwap = SharedArray[i].nFileAllTimeAccepts < SharedArray[i+1].nFileAllTimeAccepts;
				break;
			case SHARED_SORT_PRIORITY:
				//Very low priority is define equal to 4 ! Must adapte sorting code
				if(SharedArray[i].nFilePriority == 4)
				{
					if(SharedArray[i+1].nFilePriority == 4)
						bSwap = false;
					else
						bSwap = true;
				}        
				else
					if(SharedArray[i+1].nFilePriority == 4)
					{
						if(SharedArray[i].nFilePriority == 4)
							bSwap = true;
						else
							bSwap = false;
					}
					else
						bSwap = SharedArray[i].nFilePriority < SharedArray[i+1].nFilePriority;
				break;
			}
			if(pThis->m_Params.bSharedSortReverse)
			{
				bSwap = !bSwap;
			}
			if(bSwap)
			{
				bSorted = true;
				SharedFiles TmpFile = SharedArray[i];
				SharedArray[i] = SharedArray[i+1];
				SharedArray[i+1] = TmpFile;
			}
		}
	}
	// Displaying
	CString sSharedList = "";
	for(int i = 0; i < SharedArray.GetCount(); i++)
	{
		char HTTPTempC[100] = "";
		CString HTTPProcessData;
		if (SharedArray[i].sFileHash == _ParseURL(Data.sURL,"hash") )
            HTTPProcessData = OutE2;
		else
            HTTPProcessData = OutE;

		HTTPProcessData.Replace("[FileName]", _SpecialChars(SharedArray[i].sFileName));
		if(SharedArray[i].sFileName.GetLength() > SHORT_FILENAME_LENGTH)
            HTTPProcessData.Replace("[ShortFileName]", _SpecialChars(SharedArray[i].sFileName.Left(SHORT_FILENAME_LENGTH)) + "...");
		else
			HTTPProcessData.Replace("[ShortFileName]", _SpecialChars(SharedArray[i].sFileName));

		sprintf(HTTPTempC, "%4.2f " + _GetPlainResString(IDS_MBYTES), SharedArray[i].lFileSize/1048576.0);
		HTTPProcessData.Replace("[FileSize]", CString(HTTPTempC));

		HTTPProcessData.Replace("[FileLink]", SharedArray[i].sED2kLink);

		sprintf(HTTPTempC, "%4.2f " + _GetPlainResString(IDS_MBYTES), SharedArray[i].nFileTransferred/1048576.0);
		HTTPProcessData.Replace("[FileTransferred]", CString(HTTPTempC));

		sprintf(HTTPTempC, "%4.2f " + _GetPlainResString(IDS_MBYTES), SharedArray[i].nFileAllTimeTransferred/1048576.0);
		HTTPProcessData.Replace("[FileAllTimeTransferred]", CString(HTTPTempC));

		sprintf(HTTPTempC, "%i", SharedArray[i].nFileRequests);
		HTTPProcessData.Replace("[FileRequests]", CString(HTTPTempC));

		sprintf(HTTPTempC, "%i", SharedArray[i].nFileAllTimeRequests);
		HTTPProcessData.Replace("[FileAllTimeRequests]", CString(HTTPTempC));

		sprintf(HTTPTempC, "%i", SharedArray[i].nFileAccepts);
		HTTPProcessData.Replace("[FileAccepts]", CString(HTTPTempC));

		sprintf(HTTPTempC, "%i", SharedArray[i].nFileAllTimeAccepts);
		HTTPProcessData.Replace("[FileAllTimeAccepts]", CString(HTTPTempC));

		HTTPProcessData.Replace("[Priority]", SharedArray[i].sFilePriority);

		HTTPProcessData.Replace("[FileHash]", SharedArray[i].sFileHash);

		uint8 upperpriority, lesserpriority;
		if(SharedArray[i].nFilePriority == 4)
		{
			upperpriority = 0;	lesserpriority = 4;
		}
		else
		if(SharedArray[i].nFilePriority == 0)
		{
			upperpriority = 1;	lesserpriority = 4;
		}
		else
		if(SharedArray[i].nFilePriority == 1)
		{
			upperpriority = 2;	lesserpriority = 0;
		}
		else
		if(SharedArray[i].nFilePriority == 2)
		{
			upperpriority = 3;	lesserpriority = 1;
		}
		else
		if(SharedArray[i].nFilePriority == 3)
		{
			upperpriority = 5;	lesserpriority = 2;
		}
		else
		if(SharedArray[i].nFilePriority == 5)
		{
			upperpriority = 5;	lesserpriority = 3;
		}
        sprintf(HTTPTempC, "%i", upperpriority);
		HTTPProcessData.Replace("[PriorityUpLink]", "hash=" + SharedArray[i].sFileHash +"&setpriority=" + CString(HTTPTempC));
        sprintf(HTTPTempC, "%i", lesserpriority);
		HTTPProcessData.Replace("[PriorityDownLink]", "hash=" + SharedArray[i].sFileHash +"&setpriority=" + CString(HTTPTempC)); 

		sSharedList += HTTPProcessData;
	}
	Out.Replace("[SharedFilesList]", sSharedList);
	Out.Replace("[Session]", sSession);

	return Out;

	EMULE_CATCH

	return "";
}

CString CWebServer::_GetGraphs(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return "";

	CString Out = pThis->m_Templates.sGraphs;

	XBMDraw drawUp, drawDown;
	drawUp.CreateImage("image_up", WEB_GRAPH_WIDTH, WEB_GRAPH_HEIGHT);
	drawDown.CreateImage("image_down", WEB_GRAPH_WIDTH, WEB_GRAPH_HEIGHT);
	CString sGraphDownload = "", sGraphUpload = "";
	int nPrevValUp = 0, nPrevValDown = 0;
	for(int i = 0; i < WEB_GRAPH_WIDTH; i++)
	{
		if(i % 4 == 0)
		{
			for(int j = 1; j < 6; j++)
			{
				drawDown.Plot(i, (WEB_GRAPH_HEIGHT / 6) * j);
				drawUp.Plot(i, (WEB_GRAPH_HEIGHT / 6) * j);
			}
		}
		int nVal;
		if(i < pThis->m_Params.PointsForWeb.GetCount())
		{
			int nI = i;
			if(pThis->m_Params.PointsForWeb.GetCount() < WEB_GRAPH_WIDTH)
				nI = WEB_GRAPH_WIDTH - pThis->m_Params.PointsForWeb.GetCount() + i;
			// download
			nVal = (pThis->m_Params.PointsForWeb[i].download * WEB_GRAPH_HEIGHT) / (theApp.glob_prefs->GetMaxGraphDownloadRate() + 4);
			if(nVal > WEB_GRAPH_HEIGHT)
				nVal = WEB_GRAPH_HEIGHT;
			drawDown.Plot(nI, nVal);
			if(i != 0)
				drawDown.Line(nI - 1, nPrevValDown, nI, nVal);
			nPrevValDown = nVal;
			// upload
			nVal = (pThis->m_Params.PointsForWeb[i].upload * WEB_GRAPH_HEIGHT) / (theApp.glob_prefs->GetMaxGraphUploadRate() + 4);
			if(nVal > WEB_GRAPH_HEIGHT)
				nVal = WEB_GRAPH_HEIGHT;
			drawUp.Plot(nI, nVal);
			if(i != 0)
				drawUp.Line(nI - 1, nPrevValUp, nI, nVal);
			nPrevValUp = nVal;
		}
	}
	CString sUp, sDown;
	drawUp.GetImage(sUp);
	drawDown.GetImage(sDown);
	Out.Replace("[GraphDownload]", sDown + drawDown.GetImageTag());
	Out.Replace("[GraphUpload]", sUp + drawUp.GetImageTag());
	Out.Replace("[TxtDownload]", _GetPlainResString(IDS_ST_ACTIVEDOWNLOAD));
	Out.Replace("[TxtUpload]", _GetPlainResString(IDS_ST_ACTIVEUPLOAD));
	CString sScale;
	int nTime = theApp.glob_prefs->GetTrafficOMeterInterval() * WEB_GRAPH_WIDTH;
	int nHour = nTime / 3600;
	int nMin = (nTime / 60) % 60;
	int nSec = nTime % 60;
	if(nHour > 0)
		sScale.Format("%s %d:%02d %s", _GetPlainResString(IDS_TIME).GetBuffer(0), nHour, nMin, _GetPlainResString(IDS_HOURS).GetBuffer(0));
	else
		sScale.Format("%s %02d:%02d %s", _GetPlainResString(IDS_TIME).GetBuffer(0), nMin, nSec, _GetPlainResString(IDS_MINS).GetBuffer(0));
	CString s1, s2;
	s1.Format("%d " + _GetPlainResString(IDS_KBYTESEC) + ", ", theApp.glob_prefs->GetMaxGraphDownloadRate() + 4);
	s2.Format("%d " + _GetPlainResString(IDS_KBYTESEC) + ", ", theApp.glob_prefs->GetMaxGraphUploadRate() + 4);
	s1 += sScale; s2 += sScale;
	Out.Replace("[MaxDownload]", s1);
	Out.Replace("[MaxUpload]", s2);

	return Out;

	EMULE_CATCH

	return "";
}

CString CWebServer::_GetAddServerBox(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return "";

	CString sSession = _ParseURL(Data.sURL, "ses");

	CString Out = pThis->m_Templates.sAddServerBox;
	if(_ParseURL(Data.sURL, "addserver") == "true")
	{
		theApp.emuledlg->serverwnd.AddServer(_ParseURL(Data.sURL, "serveraddr"),_ParseURL(Data.sURL, "serverport"),_ParseURL(Data.sURL, "servername"));
		CString resultlog = _SpecialChars(theApp.emuledlg->logtext);	//Pick-up last line of the log
		resultlog = resultlog.TrimRight('\n');
		resultlog = resultlog.Mid(resultlog.ReverseFind('\n'));
		Out.Replace("[Message]",resultlog);
	}
	else
		if(_ParseURL(Data.sURL, "updateservermetfromurl") == "true")
		{
//			if (!theApp.glob_prefs->adresses_list.IsEmpty())
			{
				theApp.emuledlg->serverwnd.UpdateServerMetFromURL(_ParseURL(Data.sURL, "servermeturl"));
				CString resultlog = _SpecialChars(theApp.emuledlg->logtext);
				resultlog = resultlog.TrimRight('\n');
				resultlog = resultlog.Mid(resultlog.ReverseFind('\n'));
				Out.Replace("[Message]",resultlog);
			}
//			else
//               Out.Replace("[Message]",_GetPlainResString(IDS_ERR_EMPTYADRESSESDAT));
		}
		else
		Out.Replace("[Message]", "");
    Out.Replace("[AddServer]", _GetPlainResString(IDS_SV_NEWSERVER));
	Out.Replace("[IP]", _GetPlainResString(IDS_SV_ADDRESS));
	Out.Replace("[Port]", _GetPlainResString(IDS_SV_PORT));
	Out.Replace("[Name]", _GetPlainResString(IDS_SW_NAME));
	Out.Replace("[Add]", _GetPlainResString(IDS_SV_ADD));
	Out.Replace("[UpdateServerMetFromURL]", _GetPlainResString(IDS_SV_MET));
	Out.Replace("[URL]", _GetPlainResString(IDS_SV_URL));
	Out.Replace("[Apply]", _GetPlainResString(IDS_PW_APPLY));

	return Out;

	EMULE_CATCH

	return "";
}
CString	CWebServer::_GetWebSearch(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return "";

	CString sSession = _ParseURL(Data.sURL, "ses");
    
	CString Out = pThis->m_Templates.sWebSearch;
	if(_ParseURL(Data.sURL, "tosearch") != "")
	{
		CString query;
		CString tosearch = _ParseURL(Data.sURL, "tosearch");
		query = "http://www.filedonkey.com/fdsearch/index.php?media=";
		query += _ParseURL(Data.sURL, "media");
		tosearch = URLEncode(tosearch);
		tosearch.Replace("%20","+");
		query += "&pattern=";
		query += _ParseURL(Data.sURL, "tosearch");
		query += "&action=search&name=FD-Search&op=modload&file=index&requestby=emule";
		Out += "\n<script language=\"javascript\">";
		Out += "\n searchwindow=window.open('"+ query + "','searchwindow');";
		Out += "\n</script>";
	}
	Out.Replace("[Session]", sSession);
	Out.Replace("[WebSearch]", _GetPlainResString(IDS_SW_WEBBASED));
	Out.Replace("[Name]", _GetPlainResString(IDS_SW_NAME));
	Out.Replace("[Type]", _GetPlainResString(IDS_TYPE));
	Out.Replace("[Any]", _GetPlainResString(IDS_SEARCH_ANY));
	Out.Replace("[Audio]", _GetPlainResString(IDS_SEARCH_AUDIO));
	Out.Replace("[Video]", _GetPlainResString(IDS_SEARCH_VIDEO));
	Out.Replace("[Other]", "Other");
	Out.Replace("[Search]", _GetPlainResString(IDS_SW_START));
	
	return Out;

	EMULE_CATCH

	return "";
}
CString CWebServer::_GetLog(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return "";

	CString sSession = _ParseURL(Data.sURL, "ses");

	CString Out = pThis->m_Templates.sLog;

	if (_ParseURL(Data.sURL, "clear") == "yes")
	{
		theApp.emuledlg->ResetLog();
	}
	Out.Replace("[Clear]", _GetPlainResString(IDS_PW_RESET));
	Out.Replace("[Log]", _SpecialChars(theApp.emuledlg->logtext));
	Out.Replace("[Session]", sSession);

	return Out;

	EMULE_CATCH

	return "";
}

CString CWebServer::_GetServerInfo(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return "";

	CString sSession = _ParseURL(Data.sURL, "ses");

	CString Out = pThis->m_Templates.sServerInfo;

	if (_ParseURL(Data.sURL, "clear") == "yes")
	{
		theApp.emuledlg->serverwnd.servermsgbox.SetWindowText("");
	}
	Out.Replace("[Clear]", _GetPlainResString(IDS_PW_RESET));
	Out.Replace("[ServerInfo]", _SpecialChars(theApp.emuledlg->serverwnd.servermsgbox.GetHyperText()->GetText()));
	Out.Replace("[Session]", sSession);

	return Out;

	EMULE_CATCH

	return "";
}

CString CWebServer::_GetDebugLog(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return "";

	CString sSession = _ParseURL(Data.sURL, "ses");

	CString Out = pThis->m_Templates.sDebugLog;

	if (_ParseURL(Data.sURL, "clear") == "yes")
	{
		theApp.emuledlg->ResetDebugLog();
	}
	Out.Replace("[Clear]", _GetPlainResString(IDS_PW_RESET));
	Out.Replace("[DebugLog]", _SpecialChars(theApp.emuledlg->debuglog));
	Out.Replace("[Session]", sSession);

	return Out;

	EMULE_CATCH

	return "";
}

CString CWebServer::_GetStats(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return "";

	CString sSession = _ParseURL(Data.sURL, "ses");

	// refresh statistics
	theApp.emuledlg->ShowStatistics();

	CString Out = pThis->m_Templates.sStats;
	Out.Replace("[Stats]", theApp.emuledlg->statisticswnd.m_Stats.ExportHTML());

	return Out;

	EMULE_CATCH

	return "";
}

CString CWebServer::_GetPreferences(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return "";

	CString sSession = _ParseURL(Data.sURL, "ses");

	CString Out = pThis->m_Templates.sPreferences;
	Out.Replace("[Session]", sSession);

	if(_ParseURL(Data.sURL, "gzip") == "true")
	{
		theApp.glob_prefs->SetWebUseGzip(true);
	}
	if(_ParseURL(Data.sURL, "gzip") == "false")
	{
		theApp.glob_prefs->SetWebUseGzip(false);
	}
	if(_ParseURL(Data.sURL, "showuploadqueue") == "true")
	{
		pThis->m_Params.bShowUploadQueue = true;
	}
	if(_ParseURL(Data.sURL, "showuploadqueue") == "false")
	{
		pThis->m_Params.bShowUploadQueue = false;
	}
	if(_ParseURL(Data.sURL, "refresh") != "")
	{
		theApp.glob_prefs->SetWebPageRefresh(atoi(_ParseURL(Data.sURL, "refresh")));
	}
	if(_ParseURL(Data.sURL, "maxdown") != "")
	{
		if(atoi(_ParseURL(Data.sURL, "maxdown"))>0)
			theApp.glob_prefs->SetMaxDownload(atoi(_ParseURL(Data.sURL, "maxdown")));
		else
			theApp.glob_prefs->SetMaxDownload(65535);
	}
	if(_ParseURL(Data.sURL, "maxup") != "")
	{
		if(atoi(_ParseURL(Data.sURL, "maxup"))>0)
			theApp.glob_prefs->SetMaxUpload(atoi(_ParseURL(Data.sURL, "maxup")));
		else
			theApp.glob_prefs->SetMaxUpload(65535);
	}
	if(_ParseURL(Data.sURL, "maxcapdown") != "")
	{
		theApp.glob_prefs->SetMaxGraphDownloadRate(atoi(_ParseURL(Data.sURL, "maxcapdown")));
	}
	if(_ParseURL(Data.sURL, "maxcapup") != "")
	{
		theApp.glob_prefs->SetMaxGraphUploadRate(atoi(_ParseURL(Data.sURL, "maxcapup")));
	}

	if(theApp.glob_prefs->GetWebUseGzip())
	{
		Out.Replace("[UseGzipVal]", "checked");
		Out.Replace("[NotUseGzipVal]", "");
	}
	else
	{
		Out.Replace("[UseGzipVal]", "");
		Out.Replace("[NotUseGzipVal]", "checked");
	}
    if(pThis->m_Params.bShowUploadQueue)
	{
		Out.Replace("[ShowUploadQueueVal]", "checked");
		Out.Replace("[HideUploadQueueVal]", "");
	}
	else
	{
		Out.Replace("[ShowUploadQueueVal]", "");
		Out.Replace("[HideUploadQueueVal]", "checked");
	}
	Out.Replace("[UseGzipForm]", _GetPlainResString(IDS_WEB_GZIP_COMPRESSION));
	Out.Replace("[UseGzipComment]", _GetPlainResString(IDS_WEB_GZIP_COMMENT));
	Out.Replace("[UseGzip]", _GetPlainResString(IDS_USE));
	Out.Replace("[NotUseGzip]", _GetPlainResString(IDS_DONT_USE));
	Out.Replace("[ShowUploadQueueForm]", _GetPlainResString(IDS_WEB_SHOW_UPLOAD_QUEUE));
	Out.Replace("[ShowUploadQueueComment]", _GetPlainResString(IDS_WEB_UPLOAD_QUEUE_COMMENT));
	Out.Replace("[ShowUploadQueue]", _GetPlainResString(IDS_SHOW));
	Out.Replace("[HideUploadQueue]", _GetPlainResString(IDS_HIDE));
	CString sRefresh; sRefresh.Format("%d", theApp.glob_prefs->GetWebPageRefresh());
	Out.Replace("[RefreshVal]", sRefresh);
	Out.Replace("[RefreshTimeForm]", _GetPlainResString(IDS_WEB_REFRESH_TIME));
	Out.Replace("[RefreshTimeComment]", _GetPlainResString(IDS_WEB_REFRESH_COMMENT));
	Out.Replace("[SpeedForm]", _GetPlainResString(IDS_SPEED_LIMITS));
	Out.Replace("[MaxDown]", _GetPlainResString(IDS_ST_ACTIVEDOWNLOAD));
	Out.Replace("[MaxUp]", _GetPlainResString(IDS_ST_ACTIVEUPLOAD));
	Out.Replace("[SpeedCapForm]", _GetPlainResString(IDS_CAPACITY_LIMITS));
	Out.Replace("[MaxCapDown]", _GetPlainResString(IDS_ST_ACTIVEDOWNLOAD));
	Out.Replace("[MaxCapUp]", _GetPlainResString(IDS_ST_ACTIVEUPLOAD));
	Out.Replace("[Apply]", _GetPlainResString(IDS_PW_APPLY));
	CString sT;
	int n = (int)theApp.glob_prefs->GetMaxDownload();
	if(n < 0 || n == 65535) n = 0;
	sT.Format("%d", n);
	Out.Replace("[MaxDownVal]", sT);
	n = (int)theApp.glob_prefs->GetMaxUpload();
	if(n < 0 || n == 65535) n = 0;
	sT.Format("%d", n);
	Out.Replace("[MaxUpVal]", sT);
	sT.Format("%d", theApp.glob_prefs->GetMaxGraphDownloadRate());
	Out.Replace("[MaxCapDownVal]", sT);
	sT.Format("%d", theApp.glob_prefs->GetMaxGraphUploadRate());
	Out.Replace("[MaxCapUpVal]", sT);

	return Out;

	EMULE_CATCH

	return "";
}

CString CWebServer::_GetLoginScreen(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return "";

	CString sSession = _ParseURL(Data.sURL, "ses");

	CString Out = "";

	Out += pThis->m_Templates.sLogin;

	Out.Replace("[CharSet]", _GetWebCharSet());
	Out.Replace("[eMulePlus]", _GetPlainResString(IDS_EMULE_PLUS));
	Out.Replace("[version]", CURRENT_VERSION_LONG);
	Out.Replace("[Login]", _GetPlainResString(IDS_WEB_LOGIN));
	Out.Replace("[EnterPassword]", _GetPlainResString(IDS_WEB_ENTER_PASSWORD));
	Out.Replace("[LoginNow]", _GetPlainResString(IDS_WEB_LOGIN_NOW));

	return Out;

	EMULE_CATCH

	return "";
}

CString CWebServer::_GetConnectedServer(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return "";

	CString sSession = _ParseURL(Data.sURL, "ses");

	CString HTTPTemp = "";
	char	HTTPTempC[100] = "";
	CString OutS = pThis->m_Templates.sConnectedServer;
	OutS.Replace("[ConnectedServer]", _GetPlainResString(IDS_PW_SERVER));
	OutS.Replace("[Servername]", _GetPlainResString(IDS_SL_SERVERNAME));
	OutS.Replace("[Status]", _GetPlainResString(IDS_STATUS));
	OutS.Replace("[Usercount]", _GetPlainResString(IDS_LUSERS));
	OutS.Replace("[Action]", _GetPlainResString(IDS_CONNECTING));
	OutS.Replace("[URL_Disconnect]", "/?ses=" + sSession + "&w=server&c=disconnect");
	OutS.Replace("[URL_Connect]", "/?ses=" + sSession + "&w=server&c=connect");
	OutS.Replace("[Disconnect]", _GetPlainResString(IDS_IRC_DISCONNECT));
	OutS.Replace("[Connect]", _GetPlainResString(IDS_CONNECTTOANYSERVER));
	OutS.Replace("[URL_ServerOptions]", "/?ses=" + sSession + "&w=server&c=options");
	OutS.Replace("[ServerOptions]", _GetPlainResString(IDS_EM_PREFS));
	OutS.Replace("[WebSearch]", _GetPlainResString(IDS_SW_WEBBASED));

	if (theApp.serverconnect->IsConnected()) {
		if(!theApp.serverconnect->IsLowID())
			OutS.Replace("[1]", _GetPlainResString(IDS_CONNECTED));
		else
			OutS.Replace("[1]", _GetPlainResString(IDS_CONNECTED) + " (" + _GetPlainResString(IDS_LOWID) + ")");

		CServer* cur_server = theApp.serverconnect->GetCurrentServer();
		OutS.Replace("[2]", CString(cur_server->GetListName()));

		sprintf(HTTPTempC, "%10i", pThis->m_Params.nUsers);
		HTTPTemp = HTTPTempC;												
		OutS.Replace("[3]", HTTPTemp);

	} else if (theApp.serverconnect->IsConnecting()) {
		OutS.Replace("[1]", _GetPlainResString(IDS_CONNECTING));
		OutS.Replace("[2]", "");
		OutS.Replace("[3]", "");
	} else {
		OutS.Replace("[1]", _GetPlainResString(IDS_DISCONNECTED));
		OutS.Replace("[2]", "");
		OutS.Replace("[3]", "");
	}
	return OutS;

	EMULE_CATCH

	return "";
}

// We have to add gz-header and some other stuff
// to standard zlib functions
// in order to use gzip in web pages
int CWebServer::_GzipCompress(Bytef *dest, uLongf *destLen, const Bytef *source, uLong sourceLen, int level)
{ 
	EMULE_TRY

	const static int gz_magic[2] = {0x1f, 0x8b}; // gzip magic header
	int err;
	uLong crc;
	z_stream stream = {0};
	stream.zalloc = (alloc_func)0;
	stream.zfree = (free_func)0;
	stream.opaque = (voidpf)0;
	crc = crc32(0L, Z_NULL, 0);
	// init Zlib stream
	// NOTE windowBits is passed < 0 to suppress zlib header
	err = deflateInit2(&stream, level, Z_DEFLATED, -MAX_WBITS, MAX_MEM_LEVEL, Z_DEFAULT_STRATEGY);
	if (err != Z_OK)
		return err;

	sprintf((char*)dest , "%c%c%c%c%c%c%c%c%c%c", gz_magic[0], gz_magic[1],
		Z_DEFLATED, 0 /*flags*/, 0,0,0,0 /*time*/, 0 /*xflags*/, 255);
	// wire buffers
	stream.next_in = (Bytef*) source ;
	stream.avail_in = (uInt)sourceLen;
	stream.next_out = ((Bytef*) dest) + 10;
	stream.avail_out = *destLen - 18;
	// doit
	err = deflate(&stream, Z_FINISH);
	if (err != Z_STREAM_END)
	{
		deflateEnd(&stream);
		return err;
	}
	err = deflateEnd(&stream);
	crc = crc32(crc, (const Bytef *) source ,  sourceLen );
	//CRC
	*(((Bytef*) dest)+10+stream.total_out) = (Bytef)(crc & 0xFF);
	*(((Bytef*) dest)+10+stream.total_out+1) = (Bytef)((crc>>8) & 0xFF);
	*(((Bytef*) dest)+10+stream.total_out+2) = (Bytef)((crc>>16) & 0xFF);
	*(((Bytef*) dest)+10+stream.total_out+3) = (Bytef)((crc>>24) & 0xFF);
	// Length
	*(((Bytef*) dest)+10+stream.total_out+4) = (Bytef)( sourceLen  & 0xFF);
	*(((Bytef*) dest)+10+stream.total_out+5) = (Bytef)(( sourceLen >>8) & 0xFF);
	*(((Bytef*) dest)+10+stream.total_out+6) = (Bytef)(( sourceLen >>16) &	0xFF);
	*(((Bytef*) dest)+10+stream.total_out+7) = (Bytef)(( sourceLen >>24) &	0xFF);
	// return  destLength
	*destLen = 10 + stream.total_out + 8;
	return err;

	EMULE_CATCH

	return -1;
}

bool CWebServer::_IsLoggedIn(ThreadData Data, long lSession)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return "";

	// remove expired sessions
	for(int i = 0; i < pThis->m_Params.Sessions.GetSize();)
	{
		CTimeSpan ts = CTime::GetCurrentTime() - pThis->m_Params.Sessions[i].startTime;
		if(ts.GetTotalSeconds() > SESSION_TIMEOUT_SECS)
			pThis->m_Params.Sessions.RemoveAt(i);
		else
			i++;
	}
	// find our session
	// i should have used CMap there, but i like CArray more ;-)
	for(int i = 0; i < pThis->m_Params.Sessions.GetSize(); i++)
	{
		if(pThis->m_Params.Sessions[i].lSession == lSession && lSession != 0)
		{
			// if found, also reset expiration time
			pThis->m_Params.Sessions[i].startTime = CTime::GetCurrentTime();
			return true;
		}
	}

	EMULE_CATCH

	return false;
}

bool CWebServer::_RemoveSession(ThreadData Data, long lSession)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return "";

	// find our session
	for(int i = 0; i < pThis->m_Params.Sessions.GetSize(); i++)
	{
		if(pThis->m_Params.Sessions[i].lSession == lSession && lSession != 0)
		{
			pThis->m_Params.Sessions.RemoveAt(i);
			return true;
		}
	}

	EMULE_CATCH

	return false;
}


CString CWebServer::_GetStringHash(uchar* hash)
{
	EMULE_TRY

	CString sFileHash; CString sHashByte;
	for (uint16 i = 0; i < 16; i++)
	{
		sHashByte.Format("%02x", hash[i]);
		sFileHash += sHashByte;
	}
	return sFileHash;

	EMULE_CATCH

	return "";
}

bool CWebServer::_GetFileHash(CString sHash, uchar *FileHash)
{
	EMULE_TRY

	char hex_byte[3];
	int byte;
	hex_byte[2] = '\0';
	for (int i = 0; i < 16; i++) 
	{
		hex_byte[0] = sHash.GetAt(i*2);
		hex_byte[1] = sHash.GetAt((i*2 + 1));
		sscanf(hex_byte, "%02x", &byte);
		FileHash[i] = (uchar)byte;
	}
	return true;

	EMULE_CATCH

	return false;
}

CString	CWebServer::_GetPlainResString(UINT nID, bool noquot)
{
	EMULE_TRY

	CString sRet = GetResString(nID);
	sRet.Replace("&", "");
	if(noquot)
	{
        sRet.Replace("'", "\\'");
		sRet.Replace("\n", "\\n");
	}
	return sRet;

	EMULE_CATCH

	return "";
}

// EC + kuchin
CString	CWebServer::_GetWebCharSet()
{
	EMULE_TRY

	switch (theApp.glob_prefs->GetLanguageID())
	{
		case MAKELANGID(LANG_POLISH,SUBLANG_DEFAULT):				return "windows-1250";
		case MAKELANGID(LANG_RUSSIAN,SUBLANG_DEFAULT):				return "windows-1251";
		case MAKELANGID(LANG_GREEK,SUBLANG_DEFAULT):				return "ISO-8859-7";
		case MAKELANGID(LANG_HEBREW,SUBLANG_DEFAULT):				return "ISO-8859-8";
		case MAKELANGID(LANG_KOREAN,SUBLANG_DEFAULT):				return "EUC-KR";
		case MAKELANGID(LANG_CHINESE,SUBLANG_CHINESE_SIMPLIFIED):	return "GB2312";
		case MAKELANGID(LANG_CHINESE,SUBLANG_CHINESE_TRADITIONAL):	return "Big5";
		case MAKELANGID(LANG_LITHUANIAN,SUBLANG_DEFAULT):			return "windows-1257";
		case MAKELANGID(LANG_ROMANIAN,SUBLANG_DEFAULT):				return "windows-1250";
	}

	EMULE_CATCH

	// Western (Latin) includes Catalan, Danish, Dutch, English, Faeroese, Finnish, French,
	// German, Galician, Irish, Icelandic, Italian, Norwegian, Portuguese, Spanish and Swedish
	return "ISO-8859-1";
}
